<?php
// Header + Footer
$site_title = "QupZilla - Ľahký multiplatformový prehliadač";
$qupzilla = "QupZilla";
$header_description = "Webový prehliadač";

$menu_home = "Domov";
$menu_screenshots = "Screenshoty";
$menu_download = "Download";
$menu_faq = "FAQ";
$menu_about = "Zapojte sa";
$footer_site = "www.qupzilla.com";

//Home Page
$actual_version = "Aktuálna verzia";
$actual_version_text = "Aktuálna verzia QupZilly je z ". $qupzilla_version_date . 
					   " jedná sa o verziu " . $qupzilla_version . ". Zmeny môžete nájsť v changelogu. <br/>Neváhajte a <a href=\"download\">sťahujte</a> ešte dnes!";
$actual_version_button = "DOWNLOAD";
$older_versions = "Changelog";
$older_versions_text = "Zaujímá Vás, čo nové se stalo od tejto verzie po túto? <br/><br/>Kompletný <a>changelog</a> všetkých verzií je možné nájsť na githubu!";
$older_versions_button = "Changelog";
$reportbug = "Nahlásiť chybu";
$reportbug_text = "Ak ste našli chybu alebo máte len nápad na <a>vylepšenie</a>, tak práve toto je miesto pre Vás!<br/> Váš problém sa bude riešiť čo najskôr to bude možné!";
$reportbug_button = "Nahlásiť";
$technology = "Technológia";
$technology_text = "QupZilla je moderný webový prehliadač založený na jadre WebKitu a frameworku Qt. WebKit zaručí <a>rýchle</a> zobrazenie stránok a Qt funkčnosť na všetkých
				   majoritných platformách.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Nativný vzhľad";
$looknfeel_text = "QupZilla používá nativný štýl widgetov na väčšine Linuxových desktopových prostredí. Taktiež používa ikony z aktívnej desktopovej sady ikon. Pokiaľ Vám príde nativný vzhľad príliš nudný alebo s ním máte nejaké problémy, vždy je tu možnosť zmeniť tému aplikácie.";
$library_header = "Spoločná knihovňa";
$library_text = "QupZilla zjednocuje záložky, históriu a rss čítačku do jedného prehľadného okna. Už nikdy viacej okien, QupZilla používá len jedno!<br/>
S integrovanou čítačkou rss kanálov môžete byť informovaný aktualitami z Vašich obľúbených stránok. QupZilla tiež zvládne importovať záložky z iných prehliadačov.";
$adblock_header = "Integrovaný AdBlock";
$adblock_text = "Tiež nenávidíte stránky plné reklam? Zbytočne Vás zdržujú pri načítavaní stránok? Jediná vec, ktorú musíte s QupZillou urobiť, je aktualizovať EasyList alebo pridať Vaše vlastné pravidlá a môžete surfovať bez reklam.";
$speeddial_header = "Rychlá voľba (Speed dial)";
$speeddial_text = "Tento populárny doplnok je konečne dostupný aj pre uživateľov QupZilly! Teraz máte rychlý prístup k obľúbeným záložkam na jednej stránke. Samozrejmosťou je plná podpora usporiadania pretiahnutím a načítania náhľadov stránok.";
$devnews_header = "Vývojové novinky";
$feed_loading = "Nahrávam...";


// Download Page
$other_linux = "Ostatné verzie Linuxu";
$source_code = "Zdrojový kód";
$choose_os = "Zvoľte prosím operačný systém";
$windows_text = "32bit a 64bit Windows inštalátor je možné stiahnuť kliknutím na odkazy uvedené nižšie";
$ubuntu_text = "Ubuntu Precise, Oneiric, Natty a Maverick užívatelia môžu nainštalovať QupZillu zadáním týchto príkazov";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "pre vývojovú verziu:";
$packages_text = "32bit a 64bit balíčky";
$tarballs_text = "32bit a 64bit predkompilované tarbally";
$qupzilla_for_text = "QupZillu pre";
$can_be_downloaded = "je môžné stiahnuť kliknutím na odkazy uvedené nižšie";
$source_text = "Zdrojový kód môžete získať klonovaním repozitárov (pokiaľ máte nainštalovaný git)";
$source_text2 = "Môžete sa tiež pozrieť na zdrojový kód online alebo ho stiahnuť v zip archíve";
$view_source = "zobraziť kód na github.com";
$download_snapshot = "stiahnuť aktuálny zdrojový kód";

// Contribute Page
$contribute_to = "Prispieť do QupZilly";
$contribute_text = "QupZilla je open source projekt, Vaša pomoc je vítaná! Veľmi rád príjmem patche, ktoré mi pošlete. Úprava zdrojových kódov však nie je jediný spôsob ako pomôcť, môžete preložiť QupZillu do Vášho jazyku alebo se len podeliť so svojimi priateľmi. Každú, i malú pomoc si veľmi cením!";
$getting_source = "Získanie zdrojového kódu";
$getting_source_text1 = "Najjednoduchší spôsob ako získať zdrojový kód QupZilly je klonovať repozitár z githubu. Stači spustiť tento príkaz";
$getting_source_text2 = "a môžete sa pustiť do vývoja. Vaše patche mi môžete posielať e-mailom alebo priamo na githube.";
$translating = "Preklad do iných jazykov";
$translating_text = "Ďalší spôsob ako prispieť je pridať alebo upraviť existujúce preklady. Pokiaľ chcete pridať doposiaľ neobsiahnutý preklad, môžete si buď vygenerovať prekladový súbor alebo ma kontaktovať a ja to urobím za Vás. Vaše preklady mi opäť môžete posielať e-mailom alebo na githubu.";
$translating_moreinfo = "Viacej informácií o preklade <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>zde</a> [wiki na githubu]";
$creators = "Kto vytvára QupZillu?";
$creators_text = "Správcom projektu a hlavným vývojárom je český študent <b>David Rosca</b> (nowrep).<br/>
Ostatní okrem programovania tiež prispievajú prekladaním a podporovaním QupZilly. Celý zoznam prispievateľov je možné nájsť <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>tu</a> [na githubu].<br/><br/>Tiež sa môžete pripojiť na IRC kanál <b>#qupzilla</b> na irc.freenode.net a komunikovať s ľuďmi zapojenými do QupZilly.";
$share_with_friends = "Zdielať s priateľmi!";
$share_with_friends_text = "Máte radi QupZillu? Zdielajte ju teda s Vašimi priateľmi!";
?>
