<?php
// Header + Footer
$site_title = "QupZilla - 輕巧的跨平台瀏覽器";
$qupzilla = "QupZilla";
$header_description = "網頁瀏覽器";

$menu_home = "首頁";
$menu_screenshots = "螢幕擷圖";
$menu_download = "下載";
$menu_faq = "常見問答";
$menu_about = "貢獻人員";
$footer_site = "www.qupzilla.com";
$translated_by = "由Wu Cheng-Hong翻譯此頁"; // "Page translated by <translator's name>"

//Home Page
$actual_version = "當前版本";
$actual_version_text = "當前版本是來自 ". $qupzilla_version_date . " 且它的版本為 " . $qupzilla_version . "。  在更新日誌(changelog)裡檢視更新內容 <br/>不要再猶豫了!現在就<a href=\"download\">下載</a>吧!";
$actual_version_button = "下載";
$older_versions = "更新記錄";
$older_versions_text = "想知道這次的版本又有什麼變動嗎？ <br/>你可以在Github的 <a>更新日誌(changelog)</a> 上查看！";
$older_versions_button = "更新日誌";
$reportbug = "回報問題";
$reportbug_text = "發現了錯誤或只是<a>意見</a>要給QupZilla?<br/>
                   請開啟Github的Issue並幫助我們吧。";
$reportbug_button = "現在回報";
$technology = "技術";
$technology_text = "QupZilla是一款基於WebKit核心以及Qt Framework的強大瀏覽器。WebKit提供了<a>快速的</a> 
                    瀏覽體驗且Qt使得此程式可以在各種主流平台上運行。";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "原生的外觀";
$looknfeel_text = "QupZilla的原生視窗工具外觀是使用Linux環境底下的主題。同時桌面圖示也是可以變動的。如果你覺得主題太遜了或是使用上有問題，就換成其他主題吧！";
$library_header = "整合資料庫";
$library_text = "QupZilla將書籤、記錄以及RSS閱讀器放在同一個管理工具。不需要再被一堆亂七八糟的視窗影響了，QupZilla只需要一個工具就搞定了！<br/>
                 有了內建完整的RSS閱讀器，您可以與最喜愛的網站持續更新。QupZilla也可以從其他瀏覽器中匯入書籤。";
$adblock_header = "完整的廣告封鎖";
$adblock_text = "覺得充滿廣告的網站很遜？這些網站正在浪費你的頻寬及時間？您只需要更新QupZilla的EasyList或甚至加入自己定義的規則，然後好好享受沒有廣告的瀏覽。";
$speeddial_header = "快速撥號";
$speeddial_text = "這個熱門的擴充功能也可以在QupZilla上使用了！您可以直接在新分頁中檢視並開啟您最喜歡的網頁。不需多說，它也支援拖曳功能及網頁預覽。";
$devnews_header = "開發最新消息";
$feed_loading = "讀取中...";

// Download Page
$other_linux = "其他的Linux";
$source_code = "原始碼";
$choose_os = "選擇您所使用的作業系統";
$windows_text = "點擊下面的連結下載32位元及64位元的Windows的安裝包";
$ubuntu_text = "Ubuntu的Precise(12.04)、Oneiric(11.10)、Natty(11.04)以及Maverick(10.10)使用者可以藉由下列指令安裝QupZilla";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "開發中版本：";
$packages_text = "32位元以及64位元的套件包";
$tarballs_text = "32位元以及64位元的預編譯壓縮檔(tarballs)";
$qupzilla_for_text = "QupZilla 的";
$can_be_downloaded = "版本可以點擊以下連結來下載它們。";
$source_text = "您可以從Repo中取得原始碼(如果您已經安裝git的話)";
$source_text2 = "您也可以在線上觀看並下載ZIP壓縮檔";
$view_source = "在github.com觀看原始碼";
$download_snapshot = "下載當前的快照";

// Contribute Page
$contribute_to = "幫助QupZilla";
$contribute_text = "QupZilla是一個開放原始碼的計畫，所以我們很歡迎您的參與。我們會樂意接受您提供的補釘包。當然您想參與不一定要懂寫程式 
                    寫程式並不是幫助我們唯一的方法：您可以幫忙翻譯QupZilla成您所使用的語言或是將QupZilla介紹給朋友。記住！即使是小小地動作也可以給我們很大的幫助！";
$getting_source = "取得原始碼";
$getting_source_text1 = "取得QupZilla的原始碼最簡單的方法就是從Github上複製下來。您可以藉由這些步驟取得它們";
$getting_source_text2 = "然後就可以開始改寫它們。您可以藉由電子郵件或是Github將補釘包提交給我們。";
$translating = "翻譯成您所使用的語言";
$translating_text = "另一個幫助我們的方法是提供我們QupZilla的翻譯或是協助改進翻譯。如果您想要增加新的語言，您可以手動新增翻譯文件，您也可以聯絡我們，我將協助您完成。您可以藉由電子郵見或是Github將翻譯文件提交給我們。";
$translating_moreinfo = "更多關於翻譯的資訊：<a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>點擊這裡吧！</a> [github上的wiki]";
$creators = "誰創造了QupZilla？";
$creators_text = "這個計畫的擁有者、維護者以及主要開發者為捷克的學生<b>David Rosca</b>(nowrep)。<br/>
除了程式的撰寫之外，其他貢獻者也為此計畫翻譯或支持。完整的貢獻人員名單請參考<a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>這裡</a> [at github]。<br/><br/>您也可以加入IRC頻道<b>#qupzilla</b> at irc.freenode.net和參與此計畫的我們聊聊天。";
$share_with_friends = "和您的朋友分享！";
$share_with_friends_text = "喜歡QupZilla？那就跟你的朋友分享它吧！";
?>
