<?php
// Header + Footer
$site_title = "QupZilla - Navegador multiplataforma ligero";
$qupzilla = "QupZilla";
$header_description = "Navegador Web";

$menu_home = "Inicio";
$menu_screenshots = "Capturas de pantalla";
$menu_download = "Descargar";
$menu_reportbug = "Informar de un fallo";
$menu_about = "Contribuir";
$footer_site = "www.qupzilla.com";
$translated_by = ""; // "Página traducida por Jorge Sevilla"

//Home Page
$actual_version = "Versión actual";
$actual_version_text = "La versión actual es del ". $qupzilla_version_date .
					   " y es la versión " . $qupzilla_version . ". Novedades en la lista de cambios. <br/>¡No dude en <a href=\"download\">descargarla</a> ahora mismo!";
$actual_version_button = "DESCARGAR";
$older_versions = "Lista de cambios";
$older_versions_text = "¿Desea saber qué ha cambiado desde la anterior versión hasta la actual? <br/>¡Puede encontrarlo todo en la <a>lista de cambios</a> de github!";
$older_versions_button = "Lista de cambios";
$reportbug = "Informar de un fallo";
$reportbug_text = "¿Ha encontrado un fallo o tiene una sugerencia para <a>mejorar</a> QupZilla?<br/>
                  Por favor, abra un informe en el seguimiento de fallos de github.";
$reportbug_button = "Informar ahora";
$technology = "Tecnología";
$technology_text = "QupZilla es un navegador web moderno basado en Webkit core y Qt Framework. WebKit garantiza una navegación <a>rápida</a> 
                    y Qt la disponibilidad en las principales plataformas.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Apariencia nativa";
$looknfeel_text = "QupZilla utiliza un estilo de widgets nativo en los principales entornos de escritorio Linux. También utiliza los iconos del tema de iconos activo del escritorio. Si encuentra los temas demasiado aburridos o tiene algún problema con ellos, siempre puede probar otros temas.";
$library_header = "Biblioteca unificada";
$library_text = "QupZilla unifica los marcadores, el historial y el lector rss en una sola ventana organizada. ¡No a la múltiples ventanas, QupZilla utiliza sólo una!<br/>
Con el lector rss integrado, puede estar al día de sus sitios favoritos. QupZilla también puede importar los marcadores desde otros navegadores.";
$adblock_header = "Bloqueador de publicidad integrado";
$adblock_text = "¿Está cansado de sitios web repletos de publicidad? ¿Consumen éstos su ancho de banda y su tiempo? Lo único que necesita con QupZilla es actualizar Easylist o añadir sus propias reglas para empezar a navegar libre de publicidad.";
$speeddial_header = "Marcación rápida";
$speeddial_text = "¡Esta popular extensión está finalmente disponible para los usuarios de QupZilla! Puede acceder a sus páginas favoritas tan rápido como desee al abrir una pestaña nueva. No es necesario decir que soporta arrastrar y soltar, además de mostrar las miniaturas de las páginas.";
$devnews_header = "Noticias de desarrollo";
$feed_loading = "Cargando...";

// Download Page
$other_linux = "Otro Linux";
$source_code = "Código fuente";
$choose_os = "Por favor, elija su sistema operativo";
$windows_text = "Los instaladores de Windows 32bit y 64bit pueden descargarse haciendo click en los enlaces de abajo";
$ubuntu_text = "Los usuarios de Precise, Oneiric, Natty y Maverick pueden instalar QupZilla ejecutando estos comandos:";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "Para la versión en desarrollo:";
$packages_text = "Paquetes de 32bit y 64bit";
$tarballs_text = "Archivos tar precompilados en 32bit y 64bit";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "pueden descargarse haciendo click en los enlaces de abajo";
$source_text = "Puede obtener el código fuente clonando el repositorio (si tiene git instalado)";
$source_text2 = "También puede verlo en línea y descargarlo en un archivo zip";
$view_source = "ver el código fuente en github.com";
$download_snapshot = "descargar el snapshot actual";

// Contribute Page
$contribute_to = "Contribuir con QupZilla";
$contribute_text = "¡QupZilla es un proyecto de código abierto, por lo que su ayuda es bienvenida! Estaré encantado de incluir los parches que reciba. Pero modificar el 
                    código no es la única forma de ayudar: puede traducir QupZilla a su idioma o simplemente compartirlo con sus amigos. ¡Recuerde, toda ayuda (por pequeña que sea) es altamente apreciada!";
$getting_source = "Obtener el código fuente";
$getting_source_text1 = "La forma más sencilla de obtener el código fuente de QupZilla es clonando el repositorio de github. Puede hacerlo ejecutando este comando";
$getting_source_text2 = "y entonces puede empezar a modificar el código. Puede enviarme sus parches por correo electrónico o a github.";
$translating = "Traducir a otros idiomas";
$translating_text = "Otra forma para contribuir es añadir o mejorar las traducciones actuales. Si desea añadir un idioma nuevo, puede generar nuevas traducciones del archivo manualmente, o puede contactar conmigo y lo haré yo por usted. Puede enviarme sus traducciones por correo electrónico o a github.";
$translating_moreinfo = "Más información acerca de la traducción <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>aquí</a> [wiki en github]";
$creators = "¿Quién crea QupZilla?";
$creators_text = "El propietario del proyecto, mantenedor y principal desarrollador es el estudiante checo <b>David Rosca</b> (nowrep).<br/>
Además de con el código, otros también contribuyen traduciendo o dando soporte a QupZilla. La lista completa de contribuidores puede encontrarse <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>aquí</a> [en github].<br/><br/>También puede participar en el canal IRC <b>#qupzilla</b> en irc.freenode.net para chatear con gente involucrada con QupZilla.";
$share_with_friends = "¡Comparta con sus amigos!";
$share_with_friends_text = "¿Le gusta QupZilla? ¡Entonces compártalo con sus amigos!";
?>
