/*************************************
 * Default Theme v0.4                *
 * Author: nowrep                    *
 * Based on: Firefox Chromifox Theme *
 * Last updated: 04.11.2011          *
 *************************************/

*[html-link-look="true"]
{
    color: palette(link);
    text-decoration: underline;
}

/*MainWindow*/
#statusbar-ip-label
{
    padding-right: 5px;
}

/*NavigationBar*/
#navigation-button-back
{
    qproperty-themeIcon: "go-previous";
    qproperty-fallbackIcon: url(images/go-previous.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-back::menu-indicator
{
    background: url(images/transp.png);
}

#navigation-button-next
{
    qproperty-themeIcon: "go-next";
    qproperty-fallbackIcon: url(images/go-next.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-next::menu-indicator
{
    background: url(images/transp.png);
}

#navigation-button-stop
{
    qproperty-themeIcon: "process-stop";
    qproperty-fallbackIcon: url(images/process-stop.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-reload
{
    qproperty-themeIcon: "view-refresh";
    qproperty-fallbackIcon: url(images/view-refresh.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-home
{
    qproperty-themeIcon: "go-home";
    qproperty-fallbackIcon: url(images/go-home.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-addtab
{
    qproperty-themeIcon: "list-add";
    qproperty-fallbackIcon: url(images/list-add.png);
    qproperty-iconSize: 24px 24px;
}

#navigation-button-supermenu
{
    qproperty-themeIcon: "preferences-desktop";
    qproperty-fallbackIcon: url(images/preferences-desktop.png);
    qproperty-iconSize: 24px 24px;
}

/*TabWidget*/
#tabbar::tab
{
    max-height: 26px;
}

#tabbar::scroller
{
    width: 0px;
}

#tabwidget-button-opentabs
{
    qproperty-icon: url(images/tabs-list-button.png);
    qproperty-fixedsize: 20px 25px;
}

#tabwidget-button-addtab
{
    qproperty-themeIcon: "list-add";
    qproperty-fallbackIcon: url(images/list-add.png);
    qproperty-fixedsize: 22px 25px;
}

#tabwidget-button-opentabs::menu-indicator
{
    background: url(images/transp.png);
}

/*IconProvider*/
IconProvider
{
    qproperty-bookmarkIcon: url(images/star-a.png);
}

/*LocationBar*/
#locationbar
{
    border-width:4;
    padding-right: -4px;
    padding-left: -4px;
    padding-top: -2px;
    qproperty-fixedheight: 27;
    qproperty-leftMargin: 26;
}

#locationbar[secured="true"]
{
    color: black;
    background-color: #ffffbc;
}

#locationbar-bookmarkicon
{
    margin-bottom: 2px;
    qproperty-pixmap: url(images/star.png);
}

#locationbar-bookmarkicon[bookmarked="true"]
{
    qproperty-pixmap: url(images/star-a.png);
}

#locationbar-siteicon
{
    border-image: url(images/transp.png);
    qproperty-fixedsize:30px 27px;
    padding-left: 0px;
}

#locationbar-rss-icon
{
    margin-bottom:2px;
    qproperty-pixmap: url(images/rss.png);
}

#locationbar-goicon
{
    margin-bottom:2px;
    qproperty-pixmap: url(images/gotoaddress.png);
}

#locationbar-down-icon
{
    margin-bottom: 1px;
    qproperty-pixmap: url(images/navigation-dropdown.png);
}

/*BookmarksToolbar*/
#bookmarksbar QToolButton
{
    border-image: url(images/transp.png);
    border-width: 0px;
    height: 15px;
}

#bookmarksbar QToolButton::menu-indicator
{
    background: url(images/transp.png);
}

/*SideBar*/
#sidebar
{
    background: transparent;
}

/*WebSearchBar*/
#websearchbar
{
    border-width:4;
    padding-right: -4px;
    padding-top: -2px;
    qproperty-fixedheight: 27;
    qproperty-leftMargin: 30;
}

#websearchbar-searchbutton
{
    margin-bottom:2px;
    margin-right: 2px;
    qproperty-pixmap: url(images/search-icon.png);
}

#websearchbar-searchprovider-comobobox
{
    border-image: url(images/websearch-bg.png);
    padding-left:-8px;
    qproperty-fixedsize: 37px 27px;
}

#websearchbar-searchprovider-comobobox::menu-indicator
{
    background: url(images/transp.png);
}

/*SourceViewer*/
#sourceviewer-textedit
{
    border:none;
}

/*SourceViewerSearch*/
SourceViewerSearch #lineEdit[notfound="true"]
{
    background:#ff6666;
}

/*SearchToolbar*/
SearchToolBar #lineEdit[notfound="true"]
{
    background:#ff6666;
}

/*Click2Flash*/
#click2flash-frame
{
    border: 1px solid #e8e8e8;
}

#click2flash-toolbutton
{
    background: url(images/flash.png) no-repeat;
    background-position: center;
    border: none;
}

#click2flash-toolbutton:hover
{
    background: url(images/flashstart.png) no-repeat;
    background-position: center;
    border:none;
}

/*RSSManager*/
#rssmanager-frame
{
    background: white;
}

RSSManager QTabBar::tab
{
    max-width:100px;
}

/*AboutDialog*/
AboutDialog #label
{
    background:white;
}

AboutDialog #textBrowser
{
    border:none;
    margin-top: -2px; /* Workarounding rounded border on KDE */
}

/*DesktopNotification*/
DesktopNotification
{
    background: transparent;
}

DesktopNotification #frame
{
    border: 2px solid darkblue;
    border-radius: 20px;
    background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1,stop:0 #e4ecf1, stop:1 #d3e5f1);
}

DesktopNotification #heading
{
    font: bold;
    font-size: 13pt;
}

/*BrowsingLibrary*/
BrowsingLibrary #tabs
{
    qproperty-bgPixmap: url(images/library-bg-left.png);
}

BrowsingLibrary #frame
{
    background: url(images/library-bg-top.png) no-repeat;
}

BrowsingLibrary #frame_2
{
    background: url(images/library-bg-top-right.png);
}

BrowsingLibrary #searchLine
{
    background: transparent;
    border-image: url(images/library-search-bg.png);
    border-width:4;
    color:black;
}

/*Downloads*/
DownloadItem #fileName
{
    font-size: 13pt;
}

DownloadManager #list
{
    border: none;
}

/*JavaScript Dialogs*/
#jsFrame
{
    background: url(images/semi-transp.png);
}

QFrame[js-frame="true"]
{
    border: 1px solid  black;
    border-bottom: 0px;
    background-color: white;
}

QFrame[js-frame2="true"]
{
    border: 1px solid  black;
    border-top: 0px;
    background-color: #f3f3f3;
}
