<?php
// Header + Footer
$site_title = "QupZilla - Lightweight multiplatform browser";
$qupzilla = "QupZilla";
$header_description = "Web Browser";

$menu_home = "Home";
$menu_screenshots = "Screenshots";
$menu_download = "Download";
$menu_faq = "FAQ";
$menu_about = "Contribute";
$footer_site = "www.qupzilla.com";
$translated_by = ""; // "Page translated by <translator's name>"

//Home Page
$actual_version = "Current Version";
$actual_version_text = "Current version is from ". $qupzilla_version_date .
					   " and it is version " . $qupzilla_version . ". Changes in changelog. <br/>Don't hesitate and <a href=\"download\">download</a> it right now!";
$actual_version_button = "DOWNLOAD";
$older_versions = "Changelog";
$older_versions_text = "You ever wanted to know what happened from that version to this? <br/>You can find everything in <a>changelog</a> at github!";
$older_versions_button = "Changelog";
$reportbug = "Report Issue";
$reportbug_text = "Did you find a bug or you only have a suggestion how to <a>improve</a> QupZilla?<br/>
                  Please open an issue at github issue tracker.";
$reportbug_button = "Report now";
$technology = "Technology";
$technology_text = "QupZilla is modern web browser based on WebKit core and Qt Framework. WebKit guarantees <a>fast</a> 
                    browsing and Qt availability on all major platforms.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Native look'n'feel";
$looknfeel_text = "QupZilla is using native widgets style on major Linux Desktop Environments. It is also using icons from the active desktop icon theme. If you find native themes too boring or have some problems with it, you can always switch to other themes.";
$library_header = "Unified Library";
$library_text = "QupZilla unifies bookmarks, history and rss reader in one well-arranged window. No more multiple windows, QupZilla uses just one!<br/>
With the integrated rss reader, you can stay up to date with your favourite sites. QupZilla can also import bookmarks from other browsers.";
$adblock_header = "Integrated AdBlock";
$adblock_text = "Are you bored of websites full of advertisements? Are they eating your bandwidth and time? The only thing you need with QupZilla is to update EasyList or maybe add your own rules and start browsing ad free.";
$speeddial_header = "Speed Dial";
$speeddial_text = "This popular extension is finally available for QupZilla users! You can now access your favourite pages as fast as you want on one page opened in new tab. Needless to say that it fully supports drag&amp;drop and page thumbnail loading.";
$devnews_header = "Development News";
$feed_loading = "Loading...";

// Download Page
$other_linux = "Other Linux";
$source_code = "Source Code";
$choose_os = "Please choose your operating system";
$windows_text = "32bit and 64bit Windows installers can be downloaded by clicking on links below";
$ubuntu_text = "Ubuntu Precise, Oneiric, Natty and Maverick users can install QupZilla by running these commands";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "for development version:";
$packages_text = "32bit and 64bit packages";
$tarballs_text = "32bit and 64bit precompiled tarballs";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "can be downloaded by clicking on links below";
$source_text = "You can get source code by cloning repository (if you have git installed)";
$source_text2 = "You can also view it online and download it in zip archive";
$view_source = "view source on github.com";
$download_snapshot = "download current snapshot";

// Contribute Page
$contribute_to = "Contribute to QupZilla";
$contribute_text = "QupZilla is an open source project, so your help is very welcome! I will be very happy to include patches you send me. But hacking into 
                    code is not the only one way to help: You can translate QupZilla to your language or just share it with your friends. Remember, every (even small) help is highly appreciated!";
$getting_source = "Getting Source";
$getting_source_text1 = "The easiest way you can get QupZilla source is to clone it from github repository. You can do it by running this command";
$getting_source_text2 = "and then you can start hacking. You can send me your patches by e-mail or at github.";
$translating = "Translating into other languages";
$translating_text = "Another way how to contribute is to add or improve current translations. If you want add new language, you can generate new translations file manually, or you can contact me, and I will do it for you. You cand send me your translations again by e-mail or at github.";
$translating_moreinfo = "More information about translating <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>here</a> [wiki at github]";
$creators = "Who creates QupZilla?";
$creators_text = "The project owner, maintainer and main developer is Czech student <b>David Rosca</b> (nowrep).<br/>
Apart from coding, others are contributing also by making translations or supporting QupZilla. Full list of contributors can be found <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>here</a> [at github].<br/><br/>You can also join IRC channel <b>#qupzilla</b> at irc.freenode.net to chat with people involved in QupZilla.";
$share_with_friends = "Share with Friends!";
$share_with_friends_text = "Do you like QupZilla? Then share it amongst your friends!";
?>
