<?php
// Header + Footer
$site_title = "QupZilla - მარტივი მრავალპლატფორმული ბრაუზერი";
$qupzilla = "QupZilla";
$header_description = "ვებ ბრაუზერი";

$menu_home = "მთავარი";
$menu_screenshots = "სურათები";
$menu_download = "ჩამოტვირთვა";
$menu_faq = "ხ.დ.კ.";
$menu_about = "მონაწილეობა";
$footer_site = "www.qupzilla.com";
$translated_by = "გვერდის თარგმანი: ბექა არაბული"; // "Page translated by <translator's name>"

//Home Page
$actual_version = "მიმდინარე ვერსია";
$actual_version_text = "მიმდინარე ვერსიის თარიღი არის ". $qupzilla_version_date .
					   " და ეს არის " . $qupzilla_version . " ვერსია. ცვლილებები იხილეთ ცვლილებების ჟურნალში. <br/>არ მოგერიდოთ და <a href=\"download\">ჩამოტვირთეთ</a> ის ახლავე!";
$actual_version_button = "ჩამოტვირთვა";
$older_versions = "ცვლილებების ჟურნალი";
$older_versions_text = "ოდესმე მოგნდომებიათ გაგეთ თუ რა სიახლეა ამა თუ იმ ვერსიაში <br/>თქვენ ამ ყველაფრის ნახვა github-ზე განთავსებული <a>ცვლილებების ჟურნალში </a> შეგიძლიათ!";
$older_versions_button = "ცვლილებების ჟურნალი";
$reportbug = "შეცდომის შეტყობინება";
$reportbug_text = "იპოვეთ რაიმე შეცდომა ან გაქვთ რჩევა თუ როგორ შეიძლება QupZilla-ს <a>გაუმჯობესება</a>?<br/>
                  გთხოვთ შეგვატყობინეთ ამის შესახებ შეცდომების მეთვალყურე github-ზე.";
$reportbug_button = "ახლავე შეტყობინება";
$technology = "ტექნოლოგია";
$technology_text = "QupZilla არის თანამედროვე ვებ ბრაუზერი რომელიც დაფუძნებულია WebKit-ზე და Qt Framework-ზე. WebKit არის <a>სისწრაფის</a> 
                    გარანტია ხოლო Qt ყველა ძირითად პლატფორმაზე ხელმისაწვდომობის.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "სისტემური შესახედაობა";
$looknfeel_text = "QupZilla იყენებს სისტემურ ვიჯეტების სტილს  Linux-ის  ყველა ძირითადი სამუშაო გარემოსთვის. ის ასევე იყენებს თქვენი სისტემის ხატულებს. თუ სტანდარტული თემა მოსაწყენი გეჩვენეათ ან ის რაიმე პრობლემებს გიქმნით, თქვენ ყოველთვის შეგიძლიათ სხვა თემაზე გადართვა.";
$library_header = "გაერთიანებული ბიბილიოთეკა";
$library_text = "QupZilla სანიშნებს, ისტორიას და rss წამკითხველს  აერთიანებს ერთ კარგად მოწყობილ ფანჯარაში. დაივიწყეთ მრავალი გახსნილი ფანჯარა, QupZilla იყენებს მხოლოდ ერთს!<br/>
ჩადგმული rss წამკითხველის საშუალებით, თქვენ შეგიძლიათ იყოთ თქვენი საყვარელი საიტების უახლესი ამბების კურსში. QupZilla-ს ასევე შეუძლია სანიშნების შემოტანა სხვა ბრაუზერებიდან.";
$adblock_header = "ინტეგრირებული AdBlock";
$adblock_text = "დაგღალათ რეკლამებით სავსე ვებ-საიტებმა? გაკარგვინებენ ისინი გამტარუნარიანობას და დროს? QupZilla-ს გამოყენებისას თქვენ გჭირდებათ მხოლოდ EasyList-ის განახლება ან რამდენიმე თქვენი წესის დამატება რათა დაივიწყოთ რეკლამების შესახებ.";
$speeddial_header = "სწრაფი გამოძახება";
$speeddial_text = "ეს პოპულარული გაფართოება უკვე ხელმისაწვდომია QupZilla-ს მომხმარებლებისთვის! თქვენ ისე სწრაფათ შეგიძლიათ თქვენ საყვარელ საიტებზე შესვლა რამდენიც ერთი ახალი ჩანართის გასახსნელად გჭირდებათ. სწრაფ გამოძახებას ასევე აქვს გადათრევა&amp;დაგდების და ესკიზების დამზადების მხარდაჭერა.";
$devnews_header = "განვითარების სიახლეები";
$feed_loading = "იტვირთება...";

// Download Page
$other_linux = "Linux-ის სხვა დისტრიბუცია";
$source_code = "წყაროს კოდი";
$choose_os = "გთხოვთ აირჩიეთ თქვენი ოპერაციული სისტემა";
$windows_text = "Windows-ისთვის 32 და 64 ბიტიანი ინსტალატორების ჩამოტვირთვა შეგიძლიათ ქვემოთ მოცემულ ბმულებზე დაწკაპუნებით";
$ubuntu_text = "Precise-ის, Oneiric-ის, Natty-ის და Maverick-ის მოხმარებლებს QupZilla-ს დაყენება შემდეგი ბრძანებების გაშვებით შეგიძლიათ";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "დეველოპერული ვერსიისთვის:";
$packages_text = "32 და 64 ბიტიანი პაკეტები";
$tarballs_text = "32 და 64 ბიტიანი წინასწარ კომპილირებული  არქივების";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "ჩამოტვირთვა შესაძლებელია ქვემოთ მოცემულ ბმულებზე დაწკაპუნებით";
$source_text = "თქვენ შეგიძლიათ მიიღოთ წყაროს კოდის რეპოზიტორის კლონირებით (თუ git გიყენიათ)";
$source_text2 = "თქვენ ასევე შეგიძლიათ ნახოთ ის ქსელში და ჩამოტვირთოთ zip არქივი";
$view_source = "წყაროს ნახვა github.com-ზე";
$download_snapshot = "მიმდინარე ვერსიის ჩამოტვირთვა";

// Contribute Page
$contribute_to = "მიიღეთ მონაწილეობა QupZilla-ს განვითარებაში";
$contribute_text = "QupZilla არის გახსნილი კოდის პროექტი, ამიტომ თქვენი დახმარება ძალიან მნიშვნელვოვანია! მოხარული ვიქნები პროექტში თქვენს მიერ გამოგზავნილი პატჩების ჩართვით. თუმცა  
                    პროგრამირება არ არის დახმარების ერთადერთი გზა: თქვენ შეგიძლიათ თარგმნოთ QupZilla თქვენს ენაზე და გაუზიაროთ ის თქვენს მეგობრებს. დაიმახსოვრეთ, ყველანაირი (ძალიან პატარაც კი) დახმარება ძალიან ღირებულია ჩვენთვის!";
$getting_source = "წყაროს მიღება";
$getting_source_text1 = "QupZilla-ს წყაროს მიღბის ყველაზე მარტივი გზა მისი github-ის რეპოზიტორიდან კლონირებაა. ამის გაკეთება ამ ბრძანების გაშვებით შეგიძლიათ";
$getting_source_text2 = "შემდეგ შეგიძლიათ შეცვალოთ კოდი. პატჩების გამოგზავნა შეგიძლიათ ელ-ფოსტის ან github-ის გამოყენებით.";
$translating = "სხვა ენებზე თარგმნა";
$translating_text = "მონაწილეობის მიღების კიდევ ერთი გზა არის თარგმანების დამატებით ან განვითარებით. თუ ახალი ენის დამატება გსურთ, თქვენ მისი გენერირება საკუთარი ხელით შეგიძლიათ, ან შეგიძლიათ მე დამიკავშირდეთ, და თქვენთვის ამას მე გავაკეთებ. თქვენი თარგმნაის გამოგზავნა შეგიძლიათ ელ-ფოსტის ან github-ის გამოყენებით.";
$translating_moreinfo = "თარგმნის შესახებ მეტი ინფორმაციის მიღება შეგიძლიათ <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>აქ</a> [ვიკი github-ზე]";
$creators = "ვინ შექმნა QupZilla?";
$creators_text = "პროექტის მფლობელი და მთავარი დეველოპერი არის ჩეხი სტუდენტი <b>David Rosca</b> (nowrep).<br/>
პროგრამისტების გარდა, თარგმნებსა და QupZilla-ს მხარდაჭერაში სხვა ხალხიც იღებს მონაწილეობას. მონაწილეების სრული სია შეგიძლიათ იხილოთ <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>აქ</a> [at github].<br/><br/>თქვენ ასევე შეგიძლიათ შემოუერთდეთ IRC არხს <b>#qupzilla</b> irc.freenode.net-ზე რათა ისაუბროთ ხალხთან რომლებიც QupZilla-ს განვითარებაში იღებენ მონაწილეობას.";
$share_with_friends = "გაუზიარეთ მეგობრებს!";
$share_with_friends_text = "მოგწონთ QupZilla? მაშინ მოუყევით თქვენს მეგობრებს მის შესახებ!";
?>
