<?php
// Header + Footer
$site_title = "QupZilla - 轻巧的跨平台浏览器";
$qupzilla = "QupZilla";
$header_description = "网页浏览器";

$menu_home = "首页";
$menu_screenshots = "屏幕截图";
$menu_download = "下载";
$menu_faq = "常见问答";
$menu_about = "贡献人员";
$footer_site = "www.qupzilla.com";
$translated_by = "由Unink-Lio翻译此页"; // "Page translated by <translator's name>"

//Home Page
$actual_version = "当前版本";
$actual_version_text = "当前版本是来自 ". $qupzilla_version_date . " 且它的版本为 " . $qupzilla_version . "。  在更新日志(changelog)里检索更新内容 <br/>不要再犹豫了!现在就<a href=\"download\">下载</a>吧!";
$actual_version_button = "下载";
$older_versions = "更新记录";
$older_versions_text = "想知道这次的版本又有什么变动吗？ <br/>你可以在Github的 <a>更新日志(changelog)</a> 上查看！";
$older_versions_button = "更新日志";
$reportbug = "报告问题";
$reportbug_text = "发现了错误或只是<a>意见</a>要给QupZilla?<br/>
                   请开启Github的Issue并帮助我们吧。";
$reportbug_button = "现在报告";
$technology = "技术";
$technology_text = "QupZilla是一款基于WebKit核心以及Qt Framework的强大浏览器。WebKit提供了<a>快速的</a> 
                    浏览体验且Qt使得此程序可以在各种主流平台上运行。";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "原生的外观";
$looknfeel_text = "QupZilla的原生窗口工具外观是使用Linux环境下的主题。同时桌面图标也是可以变动的。如果你觉得主题太逊了或是使用上有问题，就换成其它主题吧！";
$library_header = "整合数据库";
$library_text = "QupZilla将书签、记录以及RSS阅读器放在同一个管理工具。不需要再被一堆乱七八糟的窗口影响了，QupZilla只需要一个工具就搞定了！<br/>
                 有了内建完整的RSS阅读器，您可以与最喜爱的网站持续更新。QupZilla也可以从其它浏览器中汇入书签。";
$adblock_header = "完整的广告封锁";
$adblock_text = "觉得充满广告的网站很逊？这些网站正在浪费你的带宽及时间？您只需要更新QupZilla的EasyList或甚至加入自己定义的规则，然后好好享受没有广告的浏览。";
$speeddial_header = "快速拨号";
$speeddial_text = "这个热门的扩充功能也可以在QupZilla上使用了！您可以直接在新分页中检视并开启您最喜欢的网页。不需多说，它也支持拖曳功能及网页预览。";
$devnews_header = "开发最新消息";
$feed_loading = "读取中...";

// Download Page
$other_linux = "其它的Linux";
$source_code = "原始码";
$choose_os = "选择您所使用的操作系统";
$windows_text = "点击下面的连结下载32位及64位的Windows安装包";
$ubuntu_text = "Ubuntu的Precise(12.04)、Oneiric(11.10)、Natty(11.04)以及Maverick(10.10)使用者可以用下列指令安装QupZilla";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "开发中版本：";
$packages_text = "32位以及64位的套件包";
$tarballs_text = "32位以及64位的预编译压缩文件(tarballs)";
$qupzilla_for_text = "QupZilla 的";
$can_be_downloaded = "版本可以点击以下连结来下载它们。";
$source_text = "您可以从Repo中取得原始码(如果您已经安装git的话)";
$source_text2 = "您也可以在在线观看并下载ZIP压缩档";
$view_source = "在github.com观看原始码";
$download_snapshot = "下载当前的快照";

// Contribute Page
$contribute_to = "帮助QupZilla";
$contribute_text = "QupZilla是一个开放原始码的计划，所以我们很欢迎您的参与。我们会乐意接受您提供的补钉包。当然您想参与不一定要懂写程序 
                    写程序并不是帮助我们唯一的方法：您可以帮忙翻译QupZilla成您所使用的语言或是将QupZilla介绍给朋友。记住！即使是小小地动作也可以给我们很大的帮助！";
$getting_source = "取得原始码";
$getting_source_text1 = "取得QupZilla的原始码最简单的方法就是从Github上复制下来。您可以通过这些步骤取得它们";
$getting_source_text2 = "然后就可以开始改写它们。您可以用电子邮件或是Github将补钉包提交给我们。";
$translating = "翻译成您所使用的语言";
$translating_text = "另一个帮助我们的方法是提供我们QupZilla的翻译或是协助改进翻译。如果您想要增加新的语言，您可以手动新增翻译文件，您也可以联络我们，我将协助您完成。您可以用电子邮件或是Github将翻译文件提交给我们。";
$translating_moreinfo = "更多关于翻译的信息：<a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>点击这里吧！</a> [github上的wiki]";
$creators = "谁创造了QupZilla？";
$creators_text = "这个计划的拥有者、维护者以及主要开发者为捷克的学生<b>David Rosca</b>(nowrep)。<br/>
除了程序的撰写之外，其它贡献者也为此计划翻译或支持。完整的贡献人员名单请参考<a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>这里</a> [at github]。<br/><br/>您也可以加入IRC频道<b>#qupzilla</b> at irc.freenode.net和参与此计划的我们聊聊天。";
$share_with_friends = "和您的朋友分享！";
$share_with_friends_text = "喜欢QupZilla？那就跟你的朋友分享它吧！";
?>
