<?php
// Header + Footer
$site_title = "QupZilla - Ελαφρύς πολυπλατφορμικός περιηγητής ιστού";
$qupzilla = "QupZilla";
$header_description = "ΠΕΡΙΗΓΗΤΗΣ ΙΣΤΟΥ";

$menu_home = "Αρχική";
$menu_screenshots = "Στιγμιότυπα ";
$menu_download = "Λήψη";
$menu_faq = "FAQ";
$menu_about = "Συμμετοχή";
$footer_site = "www.qupzilla.com";
$translated_by = "Μετάφραση από: Τσιβίκη Βασίλη";

//Home Page
$actual_version = "ΤΡΕΧΟΥΣΑ ΕΚΔΟΣΗ";
$actual_version_text = "Η τρέχουσα έκδοση είναι από τις ". $qupzilla_version_date .
					   " και είναι η έκδοση " . $qupzilla_version . ". Αλλαγές στο αρχείο καταγραφής αλλαγών. <br/>Μην διστάζετε και <a href=\"download\">κάντε λήψη</a> τώρα!";
$actual_version_button = "ΛΗΨΗ";
$older_versions = "ΑΛΛΑΓΕΣ";
$older_versions_text = "Θέλατε ποτέ να μάθετε τι άλλαξε από την προηγούμενη έκδοση; <br/>Μπορείτε να βρείτε τα πάντα στο <a>αρχείο καταγραφής αλλαγών</a> στο github!";
$older_versions_button = "ΑΛΛΑΓΕΣ";
$reportbug = "ΑΝΑΦΟΡΑ";
$reportbug_text = "Βρήκατε κάποιο bug ή μήπως έχετε να κάνετε κάποια πρόταση για τη <a>βελτίωση</a> του QupZilla;<br/>
                  Παρακαλώ ανοίξτε ένα issue στον ιχνηλάτη προβλημάτων του github.";
$reportbug_button = "ΑΝΑΦΟΡΑ ΤΩΡΑ";
$technology = "ΤΕΧΝΟΛΟΓΙΑ";
$technology_text = "Ο QupZilla είναι ένας μοντέρνος περιηγητής ιστού βασισμένος στον WebKit core και το Qt Framework. Το WebKit εγγυάται <a>γρήγορη</a> 
                    περιήγηση και το Qt διαθεσιμότητα σε όλες τις μεγάλες πλατφόρμες.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Εγγενής εμφάνιση και αίσθηση";
$looknfeel_text = "Το QupZilla χρησιμοποιεί εγγενή γραφικά στοιχεία σε μεγάλα γραφικά περιβάλλοντα Linux. Χρησιμοποιεί επίσης εικονίδια από το ενεργό θέμα της επιφάνειας. Αν βρείτε το εγγενές θέμα πολύ βαρετό ή έχετε κάποια προβλήματα μαζί του, μπορείτε πάντα να αλλάξετε σε άλλα θέματα.";
$library_header = "Ενιαία Βιβλιοθήκη";
$library_text = "Το QupZilla ενοποιεί τους σελιδοδείκτες, το ιστορικό και τον αναγνώστη rss σε ένα καλά τακτοποιημένο παράθυρο. Όχι άλλα πολλαπλά παράθυρα, το QupZilla χρησιμοποιεί μόνο ένα!<br/>
Με ενσωματωμένο αναγνώστη rss, μπορείτε να είστε πάντα ενημερωμένος με τις αγαπημένες σας σελίδες. Το QupZilla μπορεί επίσης να εισάγει σελιδοδείκτες από άλλους περιηγητές.";
$adblock_header = "Ενσωματωμένο AdBlock";
$adblock_text = "Βαρεθήκατε τις σελίδες που είναι γεμάτες με διαφημίσεις; Τρώνε την ευρυζωνικότητα και τον χρόνο σας; Το μόνο που χρειάζεστε με το QupZilla είναι να ενημερώσετε την EasyList ή ίσως να προσθέσετε τους δικούς σας κανόνες και αρχίστε το σερφάρισμα χωρίς διαφημίσεις.";
$speeddial_header = "Γρήγορη κλήση";
$speeddial_text = "Αυτή η δημοφιλής επέκταση είναι επιτέλους διαθέσιμη για τους χρήστες του QupZilla! Μπορείτε πλέον να έχετε πρόσβαση στις αγαπημένες σας σελίδες όσο γρήγορα θέλετε σε μια σελίδα ανοιγμένη σε νέα καρτέλα. Περιττό να πούμε ότι υποστηρίζει πλήρως το drag&amp;drop και το φόρτωμα μικρογραφιών.";
$devnews_header = "Νέα ανάπτυξης";
$feed_loading = "Φόρτωση...";

// Download Page
$other_linux = "Άλλα Linux";
$source_code = "Πηγαίος κώδικας";
$choose_os = "Παρακαλώ επιλέξτε το λειτουργικό σας σύστημα";
$windows_text = "Προγράμματα εγκατάστασης για 32bit και 64bit Windows μπορούν να ληφθούν κάνοντας κλικ στους παρακάτω συνδέσμους";
$ubuntu_text = "Οι χρήστες Precise, Oneiric, Natty and Maverick μπορούν να εγκαταστήσουν το QupZilla τρέχοντας αυτές τις εντολές";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "για την έκδοση σε ανάπτυξη:";
$packages_text = "πακέτα 32bit και 64bit";
$tarballs_text = "32bit και 64bit precompiled tarballs";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "μπορούν να ληφθούν κάνοντας κλικ στους παρακάτω συνδέσμους";
$source_text = "Μπορείτε να πάρετε τον πηγαίο κώδικα κλωνοποιώντας το repository (αν έχετε το git εγκατεστημένο)";
$source_text2 = "Μπορείτε επίσης να τον δείτε online και να τον κατεβάσετε σε zip αρχείο";
$view_source = "προβολή κώδικα στο github.com";
$download_snapshot = "λήψη τρέχοντος στιγμιότυπου";

// Contribute Page
$contribute_to = "Συνεισφέρετε στο QupZilla";
$contribute_text = "Το QupZilla είναι ένα έργο ανοιχτού κώδικα, έτσι η βοήθεια σας είναι ευπρόσδεκτη! Θα χαρώ να συμπεριλάβω patch που μου στέλνετε. Αλλά ο προγραμματισμός 
                    δεν είναι ο μόνος τρόπος για να βοηθήσετε, μπορείτε να μεταφράσετε το QupZilla στην δική σας γλώσσα ή απλά να το μοιραστείτε με τους φίλους σας. Να θυμάστε, κάθε βοήθεια (ακόμα και μικρή) εκτιμάται ιδιαίτερα!";
$getting_source = "Λαμβάνοντας τον κώδικα";
$getting_source_text1 = "Ο ευκολότερος τρόπος να πάρετε τον κώδικα του QupZilla είναι να τον κλωνοποιήσετε από το repository του github. Μπορείτε να το κάνετε εκτελώντας την παρακάτω εντολή";
$getting_source_text2 = "και μετά μπορείτε να αρχίσετε να προγραμματίζετε. Μπορείτε να μου στείλετε τα patch σας μέσο e-mail ή στο github.";
$translating = "Μεταφράζοντας σε άλλες γλώσσες";
$translating_text = "Άλλος τρόπος για να συνεισφέρετε είναι να προσθέσετε ή να βελτιώσετε μια τρέχουσα μετάφραση. Αν θέλετε να προσθέσετε μια νέα γλώσσα, μπορείτε να δημιουργήσετε νέα αρχεία μετάφρασης χειροκίνητα, ή μπορείτε να επικοινωνήσετε μαζί μου, και θα το κάνω για σας. Μπορείτε να μου στείλετε τις μεταφράσεις σας ξανά μέσω e-mail ή στο github.";
$translating_moreinfo = "Περισσότερες πληροφορίες περί μετάφρασης <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>εδώ</a> [wiki στο github]";
$creators = "Ποιός είναι ο δημιουργός του QupZilla;";
$creators_text = "Ο ιδιοκτήτης του έργου, συντηρητής και κύριος προγραμματιστής είναι ο Τσέχος φοιτητής <b>David Rosca</b> (nowrep).<br/>
Πέρα από τον κώδικα, άλλοι συνεισφέρουν επίσης κάνοντας μεταφράσεις και υποστηρίζοντας το QupZilla. Μια πλήρης λίστα με τους συνεισφέροντες μπορεί να βρεθεί <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>εδώ</a> [στο github].<br/><br/>Μπορείτε επίσης να μπείτε στο κανάλι του IRC <b>#qupzilla</b> στο irc.freenode.net για να συνομιλήσετε με κόσμο που εμπλέκεται με το QupZilla.";
$share_with_friends = "Μοιραστείτε με φίλους!";
$share_with_friends_text = "Σας αρέσει το QupZilla; Τότε μοιραστείτε το με τους φίλους σας!";
?>
