<?php
// Header + Footer
$site_title = "QupZilla - マルチプラットフォーム対応、軽量ブラウザ";
$qupzilla = "QupZilla";
$header_description = "Web Browser";

$menu_home = "ホーム";
$menu_screenshots = "スクリーンショット";
$menu_download = "ダウンロード";
$menu_faq = "FAQ";
$menu_about = "開発に貢献";
$footer_site = "www.qupzilla.com";
$translated_by = "このページはDaiki Nodaが翻訳しました";

//Home Page
$actual_version = "最新版";
$actual_version_text = "最新版の更新が行われたのは ".  $qupzilla_version_date . "です。最新版のバージョンは " . $qupzilla_version . " です。変更点については更新履歴(changelog)を見てください。<br/>ダウンロードは<a href=\"download\">こちら</a>から！";
$actual_version_button = "DOWNLOAD";
$older_versions = "更新履歴";
$older_versions_text = "各バージョンごとの機能の変遷や今までのQupZillaの更新について知りたいですか？<br/>それらはすべてgithubの<a>changelog</a>から参照することができます。";
$older_versions_button = "Changelog";
$reportbug = "問題を報告する";
$reportbug_text = "あなたがバグを発見した場合、あるいはQupzillaがよりよく<a>発展</a>するアイデアを持っているなら<br/>ぜひgithubのissue trackerにissueを投稿してください。";
$reportbug_button = "REPORT NOW";
$technology = "テクノロジー";
$technology_text = "QupZillaはレンダリングエンジンにwebkitを使い、Qt Frameworkを用いて開発された現代的なブラウザです。Webkitは<a>高速な</a>ブラウジングをQtはマルチプラットフォームでの動作を可能にします。";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "自然なUI、自然な使い心地";
$looknfeel_text = "QupZillaはメジャーなLinuxデスクトップ環境で、ネイティブなUIを再現できます。アイコンもテーマに合わせたものとなっています。ネイティブUIが退屈、あるいは何らかの問題がある場合には様々なテーマを使用することもできます。";
$library_header = "ライブラリ機能の統合";
$library_text = "QupZillaはブックマーク、履歴、RSSリーダーを最適化された一つのウィンドウに表示させます。このライブラリがあればいくつもウィンドウを広げる必要はありません。QupZillaならたったひとつのウィンドウで完結します。<br/>
				内蔵されたRSSリーダーでお気に入りのサイトの更新情報をチェックできます。またQupZillaはブックマークのインポートに対応しています";
$adblock_header = "AdBlockをデフォルトで搭載";
$adblock_text = "Webサイトのあふれるような広告にウンザリしていませんか？それらがあなたの貴重な表示領域と時間を食いつぶしていませんか？QupZillaでEasyListやあなた独自のリストを使ってそれらの広告をブロックしましょう。";
$speeddial_header = "Speed Dial";
$speeddial_text = "ついにメジャーな拡張機能であるSpeed DialがQupZillaにも登場しました！新しくタブを開くだけであなたのお気に入りのサイトに素早くアクセスできます。もちろんサムネイルのdrag&amp;dropや表示にも対応しています。";
$devnews_header = "Development News";
$feed_loading = "読み込み中...";

// Download Page
$other_linux = "Other Linux";
$source_code = "ソースコード";
$choose_os = "あなたのお使いのOSを選択してください";
$windows_text = "32bitおよび64bitのWindows向けインストーラーは以下のリンクからダウンロードできます";
$ubuntu_text = "Precise, Oneiric、NattyそしてMaverickのユーザーは以下のコマンドをお使い下さい";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "for development version:";
$packages_text = "32bit and 64bit packages";
$tarballs_text = "32bit and 64bit precompiled tarballs";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "以下のリンクからダウンロードできます";
$source_text = "gitをインストールした環境ならリポジトリからソースをcloneできます";
$source_text2 = "You can also view it online and download it in zip archive";
$view_source = "view source on github.com";
$download_snapshot = "最新のsnapshotをダウンロードする";

// Contribute Page
$contribute_to = "QupZillaがよりよいブラウザとなるために";
$contribute_text = "QupZillaはオープンソースプロジェクトであり、あなた方の協力を歓迎します。もちろんパッチを送ってくださることは非常にうれしいです。でもコードをハッキングしてバグをつぶすことが貢献の唯一の手段ではありません。QupZillaを翻訳する。友達にQupZillaを勧める。どのような貢献も我々にとって非常にありがたいことです。";
$getting_source = "ソースを入手する";
$getting_source_text1 = "もっとも簡単にQupZillaのソースを手に入れる方法はgithubでcloneすることです。このコマンドを実行すればできます。";
$getting_source_text2 = "バグの発見やパッチはe-mailやgithubで受け付けています。";
$translating = "QupZillaを翻訳する";
$translating_text = "翻訳もQupZillaの大きな助けとなります。新しい翻訳を開始するときはまず私(作者)にお知らせください。必要な作業をいくつかします。翻訳の追加や改善はe-mailやgithubで受け付けています。";
$translating_moreinfo = "翻訳に関するより詳しい情報は<a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>ここ</a>を見て下さい。 [wiki at github]";
$creators = "QupZillaはだれが作ったのか?";
$creators_text = "プロジェクトオーナー、メンテナー、メイン開発の<b>David Rosca</b>はチェコの学生です (nowrep)。<br/>
コーディングや翻訳、サポートで貢献のあった方のリストも<a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>ここ</a>から参照できます [at github].<br/><br/>IRCチャンネルにも参加できます <b>#qupzilla</b> irc.freenode.netでQupZillaにかかわる人々とチャットしましょう。";
$share_with_friends = "友達にQupZillaをすすめる！";
$share_with_friends_text = "QupZillaを気に入っていただけましたか？ ぜひお友達とシェアしてください！";
?>
