<?php
// Header + Footer
$site_title = "QupZilla - Browser leggero e multi-piattaforma";
$qupzilla = "QupZilla";
$header_description = "Web Browser";

$menu_home = "Home";
$menu_screenshots = "Screenshot";
$menu_download = "Scarica";
$menu_faq = "FAQ";
$menu_about = "Contribuisci";
$footer_site = "www.qupzilla.com";
$translated_by = ""; // "Page translated by <Francesco Marinucci>"

//Home Page
$actual_version = "Versione corrente";
$actual_version_text = "La versione corrente è stata rilasciata il ". $qupzilla_version_date .
					   " ed è la versione " . $qupzilla_version . ". Cambiamenti nel changelog. <br/>Non esitare e <a href=\"download\">scaricalo</a> adesso!";
$actual_version_button = "DOWNLOAD";
$older_versions = "Changelog";
$older_versions_text = "Ti sei mai chiesto cosa succede tra una versione e l'altra? <br/>Puoi trovare tutto nel <a>changelog</a> su github!";
$older_versions_button = "Changelog";
$reportbug = "Apri una segnalazione";
$reportbug_text = "Hai trovato un bug o vuoi solo proporre dei suggerimenti per <a>migliorare</a> QupZilla?<br/>
                  Per favore, apri una segnalazione su github issue tracker.";
$reportbug_button = "Riporta adesso";
$technology = "Tecnologia";
$technology_text = "QupZilla è un browser web moderno basato su piattaforma WebKit e Framework Qt. WebKit garantisce una navigazione <a>veloce</a> 
                    e Qt l'utilizzabilità su tutte le maggiori piattaforme.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Aspetto nativo";
$looknfeel_text = "QupZilla utilizza lo stile nativo delle applicazioni sui principali Desktop Environment di Linux. Utilizza inoltre le icone del tema attivo. Se credi che i temi nativi siano noiosi o se ti danno qualche problema, puoi sempre scegliere tra altri temi.";
$library_header = "Libreria unificata";
$library_text = "QupZilla unifica segnalibri, cronologia e lettore di fonti in un'unica finestra. Niente più finestre multiple, QupZilla ne usa solo una!<br/>
Con il lettore di fonti integrato, puoi rimanere aggiornato sui tuoi siti preferiti. QupZilla può anche importare i segnalibri da altri browser.";
$adblock_header = "Filtro AdBlock integrato";
$adblock_text = "Sei stanco dei siti pieni di pubblicità, che consumano la tua banda ed il tuo tempo? L'unica cosa che devi fare con QupZilla è aggiornare la lista EasyList o aggiungere le tue regole personalizzate ed iniziare a navigare libero dalla pubblicità.";
$speeddial_header = "Speed Dial";
$speeddial_text = "Questa famosa estensione è finalmente disponibile per gli utenti di QupZilla! Ora puoi accedere alle tue pagine preferite velocemente come vorresti, in una pagina aperta in una nuova scheda. Inutile dire che supporta appieno il drag&amp;drop e il caricamento dell'anteprima delle pagine.";
$devnews_header = "Notizie sullo sviluppo";
$feed_loading = "Caricamento...";

// Download Page
$other_linux = "Altre distribuzioni Linux";
$source_code = "Codice sorgente";
$choose_os = "Per favore, scegli il tuo sistema operativo";
$windows_text = "I pacchetti a 32bit e 64bit per Windows possono essere scaricati cliccando sui link qui sotto";
$ubuntu_text = "Gli utenti di Ubuntu Precise, Oneiric, Natty e Maverick possono installare QupZilla eseguendo questi comandi";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "per la versione in sviluppo:";
$packages_text = "pacchetti a 32bit e 64bit";
$tarballs_text = "Archivi precompilati a 32bit e 64bit";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "possono essere scaricati cliccando sui link qui sotto";
$source_text = "Puoi ottenere i codice sorgente clonando il repository (se hai git installato)";
$source_text2 = "Puoi anche visionarlo online e scaricarlo come archivio zip";
$view_source = "visualizza sorgenti su github.com";
$download_snapshot = "scarica lo snapshot corrente";

// Contribute Page
$contribute_to = "Contribuisci a QupZilla";
$contribute_text = "QupZilla è un progetto open source, quindi il tuo aiuto è molto gradito! Sarò molto felice di includere le correzioni che mi manderai. Ma l'hacking del codice  
                    non è l'unico modo per aiutare: puoi tradurre QupZilla nella tua lingua o semplicemente condividerlo con i tuoi amici. Ricorda, ogni aiuto (anche piccolo) è molto apprezzato!";
$getting_source = "Ottenere i sorgenti";
$getting_source_text1 = "Il modo più semplice per ottenere i sorgenti di QupZilla è clonarli dal repository github. Puoi farlo eseguendo questo comando";
$getting_source_text2 = "e poi puoi iniziare l'hacking. Puoi inviarmi le correzioni via e-mail o su github.";
$translating = "Tradurre in atre lingue";
$translating_text = "Un altro modo per contribuire è aggiungere traduzioni o migliorare quelle attuali. Se vuoi aggiungere una nuova traduzione, puoi generare manualmente il nuovo file di traduzione, o puoi contattarmi e lo farò io per te. Puoi inviarmi anche le traduzioni via e-mail o su github.";
$translating_moreinfo = "Maggiori informazioni sulla traduzione <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>qui</a> [wiki at github]";
$creators = "Chi crea QupZilla?";
$creators_text = "Il proprietario, mantenitore e sviluppatore principale del progetto è lo studente Ceco <b>David Rosca</b> (nowrep).<br/>
Oltre alla programmazione, anche altre persone contribuiscono a QupZilla, traducendolo o supportandolo. La lista completa dei collaboratori può essere trovata <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>qui</a> [su github].<br/><br/>Puoi anche collegarti al canale IRC <b>#qupzilla</b> su irc.freenode.net per parlare con le persone coinvolte in QupZilla.";
$share_with_friends = "Dillo ai tuoi amici!";
$share_with_friends_text = "Ti piace QupZilla? Allora condividilo con i tuoi amici!"
?>
