<?php
// Header + Footer
$site_title = "QupZilla - Lichtgewicht multiplatform-browser";
$qupzilla = "QupZilla";
$header_description = "Webbrowser";

$menu_home = "Home";
$menu_screenshots = "Screenshots";
$menu_download = "Download";
$menu_faq = "FAQ";
$menu_about = "Draag bij";
$footer_site = "www.qupzilla.com";
$translated_by = "Pagina vertaald door Heimen Stoffels";

//Home Page
$actual_version = "Actuele versie";
$actual_version_text = "Actuele versie is uitgebracht op ". $qupzilla_version_date .
					   " en dit is versie " . $qupzilla_version . ". Veranderingen staan in het wijzigingslog. <br/>Twijfel niet en <a href=\"download\">download</a> deze onmiddelijk!";
$actual_version_button = "DOWNLOAD";
$older_versions = "Wijzigingslog";
$older_versions_text = "Wilde u ooit al eens weten wat er gebeurd is vanaf die versie tot aan deze? <br/>U kunt alles vinden in het <a>wijzigingslog</a> op Github!";
$older_versions_button = "Wijzigingslog";
$reportbug = "Rapporteer probleem";
$reportbug_text = "Heeft u een bug gevonden of heeft u slechts een suggestie om QupZilla <a>te verbeteren</a>?<br/>
                  Open dan alstublieft een ticket op de Github-probleemtracker.";
$reportbug_button = "Rapporteer nu";
$technology = "Technologie";
$technology_text = "QupZilla is een moderne webbrowser gebaseerd op WebKit-core en Qt Framework. WebKit garandeert <a>snel</a> 
                    browsen en Qt-beschikbaarheid op alle grote platformen.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Geintegreerde look'n'feel";
$looknfeel_text = "QupZilla gebruikt de natieve widgets-stijl op grote Linux Bureaublad-omgevingen. Het gebruikt tevens pictogrammen van het huidige systeempictogramthema. Indien u natieve thema's te saai vindt of enige problemen hebt ermee, kunt u altijd een ander thema kiezen.";
$library_header = "Samenhangende bibliotheek";
$library_text = "QupZilla hangt bladwijzers, geschiedenis en RSS samen in een goed georganiseerd venster. Geen verschillende venster meer, QupZilla gebruikt er slechts 1!<br/>
Met de geintegreerde RSS-lezer kunt u up-to-date blijven van uw favoriete sites. QupZilla kan tevens bladwijzers importeren van andere browsers.";
$adblock_header = "Geïntegreerde AdBlock";
$adblock_text = "Bent u het zat dat sites vol reclame staan? Eten ze uw bandbreedte en tijd op? Het enige wat u hoeft te doen met QupZilla is EasyList updaten of misschien uw eigen regels creëren en begin advertentievrij te surfen.";
$speeddial_header = "Snelkiezer";
$speeddial_text = "Deze populaire extensie is eindelijk beschikbaar voor QupZilla-gebruikers! U kunt nu uw favoriete sites zo snel als u wilt bijhouden op 1 pagina op een nieuw tabblad. Wellicht overbodig om te zeggen dat het drag&amp;drop en pagina-voorbeeldweergave ondersteunt.";
$devnews_header = "Nieuws over de ontwikkeling";
$feed_loading = "Bezig met laden...";

// Download Page
$other_linux = "Overige Linux";
$source_code = "Broncode";
$choose_os = "Kies uw besturingssysteem";
$windows_text = "32-bit en 64-bit Windows-installaties kunnen worden gedownload door te klikken op onderstaande links";
$ubuntu_text = "Precise, Oneiric, Natty en Maverick-gebruikers kunnen QupZilla installeren door deze commando's te gebruken";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "voor ontwikkelings-versie:";
$packages_text = "32-bit en 64-bit-pakketten";
$tarballs_text = "32-bit en 64-bit voorgecompileerde tarballs";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "kunnen worden gedownload door op onderstaande links te klikken";
$source_text = "U kunt de broncode verkrijgen door de pakketbron te klonen (wanneer u Git heeft geinstalleerd)";
$source_text2 = "U kunt het tevens online bekijken en downloaden in zip-archiefformaat";
$view_source = "bekijk broncode op github.com";
$download_snapshot = "download huidig snapshot";

// Contribute Page
$contribute_to = "Draag bij aan QupZilla";
$contribute_text = "QupZilla is een open source-project dus uw hulp is zeer welkom! Ik ben erg verheugd om patches toe te passen die u me stuurt. Maar in de 
                    code duiken is niet de enige manier om te helpen: u kunt QupZilla vertalen naar uw taal of gewoon delen met uw vrienden. Onthoudt; elke (hoe klein ook) hulp wordt zeer gewaardeerd!";
$getting_source = "Verkrijgen van broncode";
$getting_source_text1 = "De makkelijste manier om QupZilla-broncode te verkrijgen is om de github-pakketbron te klonen. U kunt dit doen door het gebruik van het volgende commando";
$getting_source_text2 = "en dan kunt u beginnen met hacken. U kunt uw patches aan me opsturen via e-mail of op Github.";
$translating = "Vertalen naar andere talen";
$translating_text = "Een andere manier om bij te dragen is vertalingen toe te voegen of huidige te verbeteren. Indien u een nieuwe taal wilt toevoegen, kunt u een vertalingssjabloon creeren of u kunt mij contacteren en dan zal ik het voor u doen. U kunt vertalingen eveneens opsturen via e-mail of op Github.";
$translating_moreinfo = "Meer info over vertalen <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>hier</a> [wiki op Github]";
$creators = "Wie creërt QupZilla?";
$creators_text = "De project-eigenaar, -onderhouder en hoofdontwikkelaar is de Tjechische student <b>David Rosca</b> (nowrep).<br/>
Buiten het programmeren, dragen anderen tevens bij door het maken van vertalingen of het ondersteunen van QupZilla. Een volledige lijst van bijdragers kan <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>hier</a> worden gevonden [op github].<br/><br/>U kunt tevens deelnemen op ons IRC-kanaal <b>#qupzilla</b> op irc.freenode.net om te chatten met mensen die zich bezig houden met QupZilla.";
$share_with_friends = "Deel met uw vrienden!";
$share_with_friends_text = "Vind u QupZilla geweldig? Deel het dan met vrienden!";
?>
