<?php
// Header + Footer
$site_title = "QupZilla - Ein schlanker Web-Browser für Linux, MAC und Windows";
$qupzilla = "QupZilla";
$header_description = "Web Browser";

$menu_home = "Startseite";
$menu_screenshots = "Screenshots";
$menu_download = "Download";
$menu_faq = "FAQ";
$menu_about = "Mitmachen!";
$footer_site = "www.qupzilla.com";
$translated_by = "Deutsche Übersetzung: Jonathan Hooverman";

//Home Page
$actual_version = "Aktuelle Version";
$actual_version_text = "Die aktuelle Version " . $qupzilla_version . 
					   " ist am " . $qupzilla_version_date . " erschienen. Die Änderungen entnehmen Sie bitte dem CHANGELOG. <br/><br/>Direkt zum <a href=\"download\">DOWNLOAD</a>";
$actual_version_button = "DOWNLOAD";
$older_versions = "Changelog";
$older_versions_text = "Sie möchten wissen, welche Neuerungen und Verbesserungen von Version zu Version eingeflossen sind? <br/>Sie finden sie im <a>CHANGELOG</a> bei github!";
$older_versions_button = "Changelog";
$reportbug = "Fehlerbericht";
$reportbug_text = "Haben Sie einen Programmfehler entdeckt oder einen Vorschlag, wie QupZilla <a>verbessert</a> werden kann?<br/>
                  Bitte erstellen Sie einen Report bei github.";
$reportbug_button = "Erstellen";
$technology = "Technologie";
$technology_text = "QupZilla ist ein moderner Web Browser, der auf WebKit und dem Qt Framework basiert. WebKit ermöglicht <a>schnelles</a> 
                    Browsen, Qt die Verfügbarkeit auf allen wichtigen Plattformen.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Natürliches Erscheinungsbild";
$looknfeel_text = "QupZilla nutzt die Steuerelemente, die in den Bordmitteln der bedeutenden Linux Desktop Umgebungen enthalten sind. Darüber hinaus werden ebenfalls die Symbole des aktiven Desktop-Themas verwendet. Wenn Sie ab und zu einen Tapetenwechsel brauchen, so wechseln Sie einfach das Thema.";
$library_header = "Einheitliche Programmbibliotheken";
$library_text = "QupZilla vereinigt Lesezeichen, Verlauf und einen RSS Reader in einer übersichtlichen Oberfläche - Sie erleben keine über den Desktop verteilten Fenster mehr. QupZilla stellt alle Inhalte in einer Oberfläche dar!<br/>
Mit dem integrierten RSS Reader bleiben Sie immer auf dem neuesten Stand Ihrer Interessensschwerpunkte. QupZilla ist auch in der Lage, Lesezeichen von anderen Browsern zu importieren.";
$adblock_header = "Integrierter Werbeblocker";
$adblock_text = "Nerven Sie Webseiten voller Werbung, die Ihre Zeit und Bandbreite vergeudet? Um mit QupZilla werbefrei surfen zu können, müssen Sie lediglich die integrierte Filterliste (EasyList) aktualisieren. Natürlich können Sie auch eigene Filter definieren.";
$speeddial_header = "Speed Dial";
$speeddial_text = "Diese beliebte Erweiterung ist auch für QupZilla Nutzer verfügbar! Sie können Ihre Favoriten schnell und einfach erreichen, indem Sie eine Seite in einem neuen Tab öffnen. Es versteht sich von selbst, dass drag&amp;drop und eine Seitenvorschau vollständig unterstützt werden.";
$devnews_header = "Aktuelles";
$feed_loading = "Lade...";

// Download Page
$other_linux = "Weitere";
$source_code = "Quellcode";
$choose_os = "Bitte wählen Sie Ihr Betriebssystem aus";
$windows_text = "Das 32bit und 64bit Windows Installationsprogramm kann durch Klick auf die untenstehenden Links heruntergeladen werden";
$ubuntu_text = "Ubuntu Precise, Oneiric, Natty und Maverick Nutzer können QupZilla durch Ausführen der folgenden Befehle installieren:";
$install_run_command = "Installieren Sie QupZilla durch Ausführen dieses Befehls:";
$for_development_version = "Zum Installieren der aktuellen Entwicklungsversion führen Sie diesen Befehl aus:";
$packages_text = "32bit und 64bit Pakete";
$tarballs_text = "Vorkompilierte 32bit und 64bit TAR-Archive";
$qupzilla_for_text = "QupZilla für";
$can_be_downloaded = "bitte hier herunterladen:";
$source_text = "Sie können den Quellcode durch Klonen des Repositoriums erhalten (wenn Sie git installiert haben)";
$source_text2 = "Sie können sich den Quellcode auch online anschauen und als ZIP-Archiv herunterladen";
$view_source = "Quellcode bei github.com ansehen";
$download_snapshot = "Einen aktuellen Snapshot herunterladen";

// Contribute Page
$contribute_to = "Bei QupZilla mitarbeiten";
$contribute_text = "QupZilla ist ein Open Source Project. Ihre Unterstützung ist also herzlich willkommen! Ich freue mich sehr über die Zusendung von Programmerweiterungen und -verbesserungen. Dies ist jedoch nicht  
                    die einzige Möglichkeit der Unterstützung: Sie können QupZilla auch in Ihre Muttersprache übersetzen oder einfach nur Ihren Freunden und Bekannten weiterempfehlen. Denken Sie daran, dass auch die kleinste Hilfe hoch geschätzt wird!";
$getting_source = "Quellcode";
$getting_source_text1 = "Der einfachste Weg, den Quellcode von QupZilla zu beschaffen, ist, diesen von github zu klonen. Führen Sie dazu folgenden Befehl aus:";
$getting_source_text2 = "Nun können Sie mit der Arbeit beginnen. Bitte senden Sie mir Ihre Erweiterungen per E-Mail oder über github.";
$translating = "Übersetzung in weitere Sprachen";
$translating_text = "Eine weitere Möglichkeit der Mitarbeit ist, Übersetzungen zu erstellen oder bestehende zu verbessern. Wenn Sie eine neue Übersetzung erstellen möchten, so können Sie die dazu erforderliche Datei manuell erzeugen oder mit mir Kontakt aufnehmen. Ich werde sie Ihnen zur Verfügung stellen. Bitte übermitteln Sie mir Ihre Übersetzung per E-Mail oder über github.";
$translating_moreinfo = "Weitere Informationen zu Übersetzungen finden Sie <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>hier</a> [wiki auf github]";
$creators = "Wer entwickelt QupZilla?";
$creators_text = "Verantwortlich für dieses Projekt und dessen Pflege sowie Hauptentwickler ist der tschechische Student <b>David Rosca</b> (nowrep).<br/>
Außer der eigentlichen Programmierarbeit unterstützen weitere Mitarbeiter dieses Projekt, indem sie z.B. Übersetzungen erstellen oder QupZilla testen. Eine vollständige Liste aller Mitarbeiter befindet sich <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>hier</a> [auf github].<br/><br/>Sie können sich ebenfalls dem IRC channel <b>#qupzilla</b> auf irc.freenode.net anschließen, um mit den Leuten zu chatten, die an QupZilla beteiligt sind.";
$share_with_friends = "Mit Freunden teilen!";
$share_with_friends_text = "Mögen Sie QupZilla? Dann teilen Sie ihn mit Ihren Freunden!";
?>
