<?php
// Header + Footer
$site_title = "QupZilla - Lehký multiplatformní prohlížeč";
$qupzilla = "QupZilla";
$header_description = "Webový prohlížeč";

$menu_home = "Domů";
$menu_screenshots = "Screenshoty";
$menu_download = "Download";
$menu_faq = "FAQ";
$menu_about = "Zapojte se";
$footer_site = "www.qupzilla.com";

//Home Page
$actual_version = "Aktuální verze";
$actual_version_text = "Aktuální verze QupZilly je z ". $qupzilla_version_date . 
					   " jedná se o verzi " . $qupzilla_version . ". Změny lze nalézt v changelogu. <br/>Neváhejte a <a href=\"download\">stahujte</a> ještě dnes!";
$actual_version_button = "DOWNLOAD";
$older_versions = "Changelog";
$older_versions_text = "Zajímá Vás co nového se stalo od této verze po tuto? <br/><br/>Kompletní <a>changelogu</a> všech verzí lze nalézt  na githubu!";
$older_versions_button = "Changelog";
$reportbug = "Nahlásit chybu";
$reportbug_text = "Pokud jste našli chybu nebo máte pouze nápad na <a>vylepšení</a>, právě toto místo je pro Vás!<br/> Váš problém se bude řešit co nejdříve to bude možné!";
$reportbug_button = "Nahlásit";
$technology = "Technologie";
$technology_text = "QupZilla je moderní webový prohlížeč založený na jádře WebKitu a frameworku Qt. WebKit zaručí <a>rychlé</a> zobrazení stránek a Qt funkčnost na všech
				   majoritních platformách.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Nativní vzhled";
$looknfeel_text = "QupZilla používá nativní styl widgetů na většině Linuxových desktopových prostředí. Také používá ikony z aktivní desktopové sady ikon. Pokud Vám přijde nativní vzhled příliš nudný nebo s ním máte nějaké problémy, vždy je možnost změnit téma aplikace.";
$library_header = "Společná knihovna";
$library_text = "QupZilla sjednocuje záložky, historii a rss čtečku do jednoho přehledného okna. Už nikdy více oken, QupZilla používá pouze jedno!<br/>
S integrovanou čtečkou rss kanálů můžete být informován aktualitami z Vašich oblíbených stránek. QupZilla také zvládne importovat záložky z jiných prohlížečů.";
$adblock_header = "Integrovaný AdBlock";
$adblock_text = "Také nenávidíte stránky plné reklam? Zbytečně Vás zdržují při načítání stránek? Jediná věc kterou musíte s QupZillou udělat, je aktualizovat EasyList nebo přidat Vaše vlastní pravidla a můžete surfovat bez reklam.";
$speeddial_header = "Rychlá volba (Speed dial)";
$speeddial_text = "Tento populární doplněk je konečně dostupný i pro uživatele QupZilly! Nyní máte rychlý přístup k oblíbeným záložkám na jediné stránce. Samozřejmostí je plná podpora řazení přetažením a načítání náhledů stránek.";
$devnews_header = "Vývojové novinky";
$feed_loading = "Načítám...";


// Download Page
$other_linux = "Ostatní Linux";
$source_code = "Zdrojový kód";
$choose_os = "Zvolte prosím operační systém";
$windows_text = "32bit a 64bit Windows instalátor lze stáhnout kliknutím na odkazy níže";
$ubuntu_text = "Ubuntu Precise, Oneiric, Natty a Maverick uživatelé mohou nainstalovat QupZillu zadáním těchto příkazů";
$install_run_command = "QupZillu můžete nainstalovat zadáním tohoto příkazu";
$for_development_version = "pro vývojovou verzi:";
$packages_text = "32bit a 64bit balíčky";
$tarballs_text = "32bit a 64bit předkompilované tarbally";
$qupzilla_for_text = "QupZillu pro";
$can_be_downloaded = "lze stáhnout kliknutím na odkazy níže";
$source_text = "Zdrojový kód můžete získat klonováním repositáře (pokud máte nainstalovaný git)";
$source_text2 = "Můžete se také podívat na zdrojový kód online nebo ho stáhnout v zip archivu";
$view_source = "zobrazit kód na github.com";
$download_snapshot = "stáhnout aktuální zdrojový kód";

// Contribute Page
$contribute_to = "Přispět do QupZilly";
$contribute_text = "QupZilla je open source projekt, Vaše pomoc je tedy vítána! Velmi rád zahrnu patche, které mi pošlete. Úprava zdrojových kódů však není jediný způsob jak pomoci, můžete přeložit QupZillu do Vašeho jazyku nebo se pouze podělit se svými přáteli. Každé, i malé, pomoci si velmi cením!";
$getting_source = "Získání zdrojového kódu";
$getting_source_text1 = "Nejjednodušší způsob jak získat zdrojový kód QupZilly je klonovat repozitář z githubu. Stačí spustit tento příkaz";
$getting_source_text2 = "a můžete se pustit do vývoje. Vaše patche mi můžete posílat e-mailem nebo přímo na githubu.";
$translating = "Překlad do jiných jazyků";
$translating_text = "Další způsob jak přispět je přidat nebo upravit stávající překlady. Pokud chcete přidat dosud neobsažený překlad, můžete si buď vygenerovat překladový soubor, nebo mne kontaktovat a já to udělám za Vás. Vaše překlady mi opět můžete posílat e-mailem nebo na githubu.";
$translating_moreinfo = "Více informací o překládání <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>zde</a> [wiki na githubu]";
$creators = "Kdo vytváří QupZillu?";
$creators_text = "Správcem projektu a hlavním vývojářem je český student <b>David Rosca</b> (nowrep).<br/>
Ostatní kromě programování také přispívají překládáním a podporováním QupZilly. Celý seznam přispěvatelů lze najít <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>zde</a> [na githubu].<br/><br/>Také se můžete připojit na IRC kanál <b>#qupzilla</b> na irc.freenode.net a komunikovat s lidmi zapojenými do QupZilly.";
$share_with_friends = "Sdílet s přáteli!";
$share_with_friends_text = "Máte rádi QupZillu? Sdílejte ji tedy s Vašimi přáteli!";
?>
