<?php
// Header + Footer
$site_title = "QupZilla - Un navigateur web léger et multi-plateforme";
$qupzilla = "QupZilla";
$header_description = "Navigateur web";

$menu_home = "Accueil";
$menu_screenshots = "Screenshots";
$menu_download = "Télécharger";
$menu_faq = "FAQ";
$menu_about = "Contribuer";
$footer_site = "www.qupzilla.com";
$translated_by = "Page traduite par Jérôme Giry et Nicolas Ourceau";

//Home Page
$actual_version = "Version actuelle";
$actual_version_text = "La version actuelle date du ". $qupzilla_version_date .
                        " et la version est " . $qupzilla_version . ". Les changements sont décrits dans le changelog. <br/>N'hésitez pas à la <a  href=\"download\">télécharger</a> maintenant !";
$actual_version_button = "Télécharger";
$older_versions = "Changelog";
$older_versions_text  = "Vous avez toujours voulu savoir ce qu'il s'est passé entre deux versions ? <br/>Vous pouvez tout trouver dans le <a>changelog</a> sur GitHub !";
$older_versions_button = "Changelog";
$reportbug = "Rapporter un bug";
$reportbug_text  = "Avez-vous trouvé un bug ou voulez vous simplement proposer une idée pour <a>améliorer</a> QupZilla ?<br/>
                  Pour le rapporter, faites un rapport de bug sur GitHub.";
$reportbug_button = "Rapporter";
$technology = "Technologie";
$technology_text  = "QupZilla est un navigateur web basé sur le moteur de rendu WebKit et sur le framework Qt. WebKit garantie une navigation <a>rapide</a>
                    et Qt une disponibilité sur toutes les plateformes majeures.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Style et comportements natifs";
$looknfeel_text  = "QupZilla utilise nativement les thèmes pour les principaux bureaux Linux. Il utilise aussi les thèmes d'icônes de l'utilisateur. Si vous vous lassez du thème de base ou si vous avez des problèmes avec, vous pouvez toujours en choisir un autre.";
$library_header = "Des outils unifiés";
$library_text  = "QupZilla unifie les marque-pages, l'historique et le lecteur de flux RSS en une page bien organisée. Plus de fenètres multiples, QupZilla n'en utilise qu'une seule !<br/>
Avec l'intégration du lecteur de flux RSS, vous restez informé des mises à jour de vos sites préférés. QupZilla peut aussi importer les marque-pages des autres navigateurs.";
$adblock_header = "AdBlock";
$adblock_text  = "Les sites remplis de publicités vous ennuyent ? Ils gaspillent votre bande-passante et votre temps ? La seule chose dont vous avez besoin, grâce à QupZilla, est de mettre à jour votre EasyList, ou éventuellement d'ajouter vos propres règles de blocage, et ainsi naviguer en paix.";
$speeddial_header = "Speed Dial";
$speeddial_text  = "Cette extention populaire est finalement disponible pour les utilisateurs de QupZilla ! Vous pouvez maintenant accéder à vos sites favoris aussi vite que vous le souhaitez en ouvrant une seule page dans un nouvel onglet. Inutile de préciser que le glisser-déposer et le chargement des miniatures sont totalement supportés.";
$devnews_header = "Nouvelles du développement";
$feed_loading = "Chargement...";

// Download Page
$other_linux = "Autres distributions linux";
$source_code = "Code Source";
$choose_os = "Choisissez votre système d'exploitation";
$windows_text = "Les versions 32 et 64 bits peuvent être téléchargés en cliquant sur les liens suivants";
$ubuntu_text = "Les utilisateurs d'Ubuntu Precise, Oneiric, Natty et Maverick peuvent installer QupZilla en exécutant ces commandes";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "Pour les versions de développement:";
$packages_text = "Paquets 32 et 64 bits";
$tarballs_text = "Tarballs 32 et 64 bits précompilés";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "Peuvent être téléchargées en cliquant sur les liens suivants";
$source_text = "Vous pouvez récupérer le code source en clonant le dépôt (si vous avez installé git)";
$source_text2 = "Vous pouvez aussi le consulter en ligne et le télécharger au format zip";
$view_source = "Voir le code source sur GitHub.com";
$download_snapshot = "Télécharger la version actuelle";

// Contribute Page
$contribute_to = "Contribuer à QupZilla";
$contribute_text  = "QupZilla est un projet Open Source, donc votre aide est la bienvenue ! Je serais très heureux d'inclure les patches que vous m'enverriez. Mais coder
                     n'est pas la seule manière d'aider : vous pouvez traduire QupZilla dans votre langue ou juste en parler à vos amis. Souvenez-vous, toutes les participations (mêmes petites) sont hautement appréciées";
$getting_source = "Obtenir le fichier source";
$getting_source_text1  = "La façon la plus simple d'obtenir les sources de QupZilla est de les cloner à partir du dépôt sur GitHub. Vous pouvez le faire en utilisant cette commande";
$getting_source_text2 = "Vous pouvez ensuite commencer à coder. Vous pouvez m'envoyer vos modifications par e-mail ou sur GitHub.";
$translating = "Traductions dans d'autres langues";
$translating_text  = "Une autre façon de participer est d'ajouter ou d'améliorer les traductions existantes. Si vous voulez ajouter une nouvelle langue, vous pouvez générer un nouveau fichier de traduction manuellement, ou me contacter et je le ferai pour vous. Vous pouvez m'envoyer votre traduction à nouveau par e-mail ou sur GitHub.";
$translating_moreinfo = "Plus d'informations sur la traduction <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>ici</a> [wiki sur GitHub]";
$creators = "Qui développe QupZilla ?";
$creators_text  = "Le responsable du projet, mainteneur et développeur principal est un étudiant tchèque <b>David Rosca</b> (nowrep).<br/>
Outre le codage, d'autres contributeurs participent également en réalisant les traductions ou en soutenant QupZilla. La liste complête des contributeurs peut être trouvée <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>ici</a>  [sur GitHub].<br/><br/>Vous pouvez aussi rejoindre le canal IRC  <b>#qupzilla</b> sur irc.freenode.net pour discuter avec les gens impliqués dans QupZilla.";
$share_with_friends = "Partagez avec vos amis!";
$share_with_friends_text = "Vous aimez QupZilla ? Alors partagez-le avec vos amis !!"
?>
