<?php
// Header + Footer
$site_title = "QupZilla - lekka i wieloplatformowa przeglądarka internetowa";
$qupzilla = "QupZilla";
$header_description = "Nowoczesna Przeglądarka";

$menu_home = "Home";
$menu_screenshots = "Zrzuty";
$menu_download = "Pobierz";
$menu_faq = "FAQ";
$menu_about = "Przyczyń się";
$footer_site = "www.qupzilla.com";
$translated_by = "Page translated by Mariusz Fik"; // "Page translated by <translator's name>"

//Home Page
$actual_version = "Aktualna Wersja";
$actual_version_text = "Aktualna wersja jest z ". $qupzilla_version_date .
					   " i jest to wersja " . $qupzilla_version . ". Zmiany w changelogu. <br/>Nie wahaj się i <a href=\"download\">pobierz</a> już teraz!";
$actual_version_button = "POBIERZ";
$older_versions = "Lista Zmian";
$older_versions_text = "Czy kiedykolwiek chciałeś wiedzieć, co się zmieniło w nowej wersji? <br/>Wszystko możesz znaleźć w <a>liście zmian</a> na github!";
$older_versions_button = "Lista zmian";
$reportbug = "Zgłoś Błąd";
$reportbug_text = "Czy znalazłeś jakiś błąd lub masz tylko sugestie, jak <a>poprawić</a> QupZille?<br/>
                  Otwórz nowy wątek na trackerze QupZilli na github.";
$reportbug_button = "Zgłoś teraz";
$technology = "Technologia";
$technology_text = "QupZilla jest nowoczesną przeglądarką bazującą na silniku WebKit i Frameworku Qt. WebKit gwarantuje <a>szybkie</a> 
                    przeglądanie a Qt wieloplatformowość aplikacji.";
$technology_button = "WebKit i Qt";
$looknfeel_header = "Nowoczesny wygląd i odczucia";
$looknfeel_text = "QupZilla wykorzystuje wiele natywnych motywów dla popularnych linuksowych środowisk graficznych. Jeżeli twój natywny motyw jest zbyt nudny lub masz z nim jakieś problemy, możesz w prosty sposób go zmienić.";
$library_header = "Ujednolicone Biblioteki";
$library_text = "QupZilla łączy zakładki, historię oraz czytnik rss w jednym oknie. Nigdy więcej dla takiego rozbicia, z QupZillą wszystko będziesz mieć w jednym oknie!<br/>
Dzięki wbudowanemu czytnikowi RSS, można pozostać na bieżąco z ulubionymi stronami. QupZilla może także importować zakładki z innych przeglądarek.";
$adblock_header = "Zintegrowany AdBlock";
$adblock_text = "Czy nie denerwują się strony przeładowane reklamami?  Ładują się za długo? Jedyne co potrzebujesz w QupZill to aktualizacja EasyList lub dodanie swojich własnych reguł blokowania reklam.";
$speeddial_header = "Speed Dial";
$speeddial_text = "Ten popularny dodatek wreszcie zagościł w QupZilli! Możesz teraz uzyskać dostęp do ulubionych stron tak szybko, jak chcesz w każdej nowej karcie. Nie zapomnijmy wspomnieć że Spped Dial umożliwia zarządznie zakładkami za pomocą drag&amp;drop (przeciągnij i upuść).";
$devnews_header = "Nowości";
$feed_loading = "Ładowanie...";

// Download Page
$other_linux = "Inne z rodziny Linux";
$source_code = "Kod Źródłowy";
$choose_os = "Wybierz system operacyjny";
$windows_text = "32bit i 64bit instalator dla systemu Windows można pobrać klikając na linki poniżej";
$ubuntu_text = "Użytkownicy Ubuntu Precise, Oneric, Natty i Maverick mogą zainstalować QupZille wykonując komendy";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "wersja developerska:";
$packages_text = "Paczki 32bit i 64bit";
$tarballs_text = "32bit and 64bit precompiled tarballs";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "można pobrać klikająć w link poniżej";
$source_text = "Możesz pobrać kod źródłowy, klonując repozytorium (jeżeli ma się zainstalowany git)";
$source_text2 = "Możesz także przeglądać kod źródłowy online i pobrać go w postaci archiwum zip";
$view_source = "pokaż źródła na github.com";
$download_snapshot = "Pobierz aktualną migawkę";

// Contribute Page
$contribute_to = "Przyczyń się dla QupZilli";
$contribute_text = "QupZilla jest projektem open source, dlatego twoja pomoc jest mile widziana! Będę bardzo szczęśliwy z poprawek które mi wyślesz. 
                    Tworzenie kodu nie jest jedynym sposobem aby pomóc, można tłumaczyć QupZille na swój język lub po prostu podzielić się nią ze znajomymi. Pamiętaj, że każda (nawet mała) pomoc jest wysoko ceniona!";
$getting_source = "Kod Źródłowy";
$getting_source_text1 = "Najprostszą metodą na zdobycie kodu źródłowego jest sklonowanie repozytorium na github. Możesz to zrobić wykonując komendę";
$getting_source_text2 = "i poprostu zacząć hakować. Możesz wysyłać poprawki za pomoca e-maila lub po przez github.";
$translating = "Tłumaczenie na inne języki";
$translating_text = "Innym sposobem aby przyczynić sie dla projektu jest dodanie nowego lub poprawienie obecnego tłumaczenia. Jeżeli chcesz dodać nowe tłumaczenie, możesz wygenerowac plik tłumaczenia ręcznie, lub skontaktować się ze mną, a ja zrobie to dla Ciebie. Możesz mi wysłać swoje tłumaczenie po przez e-mial lub github.";
$translating_moreinfo = "Więcej informacji o tłumaczeniu <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>tutaj</a> [wiki na github]";
$creators = "Kto tworzy QupZillę?";
$creators_text = "Właścicielem projektu i głównym programistą jest student z Republiki Czeskiej, <b>David Rosca</b> (nowrep).<br/>
Poza kodowaniem, użytkownicy współtworzą tłumaczenia na wiele języków i wspierają QupZillę. Lista osób współtworzących dostępna jest <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>tutaj</a> [github].<br/><br/>Możesz także dołączyć do kanału IRC <b>#qupzilla</b> w sieci irc.freenode.net aby porozmawiać z osobami zaangażowanymi w rozwój QupZilli.";
// You can delete these (commented) lines after you translate $creators and $creators_text
//$contributors = "Contributors";
//$contributors_text = "Full list of contributors can be found on <a href='https://github.com/nowrep/QupZilla/blob/master/AUTHORS'>github</a>.<br/>You can also join IRC channel #qupzilla at irc.freenode.net.";
$share_with_friends = "Podziel się ze znajomymi!";
$share_with_friends_text = "Lubisz QupZillę? W takim razie podziel się nią ze swoimi znajomymi!";
?>
