<?php
// Header + Footer
$site_title = "Капзила - Лаган мултиплатформски веб прегледач";
$qupzilla = "Капзила";
$header_description = "Веб прегледач";

$menu_home = "Почетна";
$menu_screenshots = "Снимци екрана";
$menu_download = "Преузимање";
$menu_faq = "FAQ";
$menu_about = "Доприноси";
$footer_site = "www.qupzilla.com";
$translated_by = "Превод на српски: Младен Пејаковић";

//Home Page
$actual_version = "Тренутно издање";
$actual_version_text = "Тренутно издање је ". $qupzilla_version .
					   " изашло " . $qupzilla_version_date . ". Измене су у фајлу дневника измена. <br/>Не оклевајте и <a href=\"download\">преузмите</a> одмах!";
$actual_version_button = "ПРЕУЗМИ";
$older_versions = "Дневник измена";
$older_versions_text = "Да ли сте икад желели да знате шта се променило из једнога у друго издање? <br/>Све можете да пронађете у <a>дневнику измена</a> на Гитхабу!";
$older_versions_button = "Дневнк измена";
$reportbug = "Пријава проблема";
$reportbug_text = "Открили сте неку грешку или имате предлог за <a>побољшање</a> Капзиле?<br/>
                  Направите пријаву на Гитхабовом пратиоцу пријава.";
$reportbug_button = "Пријави";
$technology = "Технологија";
$technology_text = "Капзила је модеран веб прегледач заснован на Вебкит језгру и Кјут радном окружењу, што гарантује <a>брзину</a> 
                    и доступност на свим главним платформама.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Изворни изглед и понашање";
$looknfeel_text = "Капзила користи изворни стил виџета на главним Линукс окружењима радне површи. Такође користи иконе активне теме икона радне површи. Ако су вам изворне теме превише досадне или имате проблема са њима, увек можете променити тему.";
$library_header = "Јединствена библиотека";
$library_text = "Капзила уједињује обележиваче, историјат и читач РСС довода у једном добро организованом прозору. Нема више вишеструких прозора, Капзила користи само један!<br/>
Помоћу уграђеног РСС читача можете да останете у току са вашим омиљеним сајтовима. Такође Капзила може да увезе обележиваче из осталих прегледача.";
$adblock_header = "Уграђен АдБлок";
$adblock_text = "Досадили вам веб сајтови препуни реклама? Троше вам време и интернет проток? Све што треба да урадите са Капзилом је да ажурирате EasyList списак или да додате сопствене филтере и да почнете са прегледањем без реклама.";
$speeddial_header = "Брзо бирање";
$speeddial_text = "Ово популарно проширење је коначно доступно Капзилиним корисницима! Сада можете да приступите вашим омиљеним страницама веома брзо са само једном страницом у новом језичку. Наравно, подржано је превлачење и учитавање сличица.";
$devnews_header = "Новости о развоју";
$feed_loading = "Учитавање...";

// Download Page
$other_linux = "Остали Линукси";
$source_code = "Изворни кôд";
$choose_os = "Изаберите ваш оперативни систем";
$windows_text = "Можети преузети 32 битне и 64 битне Виндоуз инсталатере кликом на везе испод";
$ubuntu_text = "Precise, Oneiric, Natty и Maverick корисници могу инсталирати Капзилу покретањем ових наредби";
$install_run_command = "Можете инсталирати Капзилу помоћу ове наредбе";
$for_development_version = "за издање у развоју:";
$packages_text = "32 битни и 64 битни пакети";
$tarballs_text = "32 битни и 64 битни компилирани архиви пакета";
$qupzilla_for_text = "Пакети за";
$can_be_downloaded = "могу бити преузети кликом на везе испод";
$source_text = "Изворни кôд можете да набавите тако што ћете клонирати ризницу (ако имате инсталиран гит)";
$source_text2 = "Такође га можете прегледати на мрежи и преузети као зип архиву";
$view_source = "прегледај кôд на github.com";
$download_snapshot = "преузми тренутни снимак";

// Contribute Page
$contribute_to = "Допринесите Капзили";
$contribute_text = "Капзила је пројект отвореног кôда, па је ваша помоћ веома добродошла! Радо ћу прихватити измене које ми пошаљете. Али хаковање кôда 
                    није једини начин на који можете да помогнете, можете да преведете Капзилу на ваш језик или само је поделити са вашим пријатељима. Свака (чак и мала) помоћ је веома цењена!";
$getting_source = "Добављање изворног кôда";
$getting_source_text1 = "Најлакши начин за добављање Капзилиног изворног кôда је да га клонирате са ризнице на гитхабу. То можете да урадите помоћу ове наредбе";
$getting_source_text2 = "и хаковање може да почне! Можете ми послати ваше измене е-поштом или преко гитхаба.";
$translating = "Превођење на остале језике";
$translating_text = "Још један начин за допринос је да урадите или побољшате тренутни превод. Ако желите да додате нови језик можете да направите нови фајл превода ручно, или можете да контактирате мене, и ја ћу то да урадим за вас. Можете ми послати ваше преводе опет е-поштом или преко гитхаба.";
$translating_moreinfo = "Више информација о превођењу <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>овде</a> [вики на гитхабу]";
$creators = "Ко је аутор Капзиле?";
$creators_text = "Власник пројекта, одржавалац и главни програмер је чешки студент <b>Давид Роска</b> (David Rosca - nowrep).<br/>
Поред кодирања, и други су такође допринели тиме што преводе или подржавају Капзилу. Потпун списак доприносиоца се може наћи <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>овде</a> [на гитхабу].<br/><br/>Такође, можете се придружити ИРЦ каналу <b>#qupzilla</b> на irc.freenode.net да бисте ћаскали са људима укљученим у пројект Капзила.";
$share_with_friends = "Поделите са пријатељима!";
$share_with_friends_text = "Допада вам се Капзила? Поделите је са пријатељима!";
?>
