<?php
// Header + Footer
$site_title = "QupZilla - Легкий мультиплатформенный веб-браузер";
$qupzilla = "QupZilla";
$header_description = "Веб-браузер";

$menu_home = "На главную";
$menu_screenshots = "Скриншоты";
$menu_download = "Скачать";
$menu_faq = "FAQ";
$menu_about = "Внести вклад";
$footer_site = "www.qupzilla.com";
$translated_by = "Перевод выполнил Брежнев Олег";

//Home Page
$actual_version = "Текущая версия";
$actual_version_text = "Текущая версия от ". $qupzilla_version_date .
					   " Это версия " . $qupzilla_version . ". Просмотреть изменения можно в журнале изменений. <br/>Не стесняйтесь - <a href=\"download\">скачайте</a> прямо сейчас!";
$actual_version_button = "СКАЧАТЬ";
$older_versions = "Журнал изменений";
$older_versions_text = "Вы когда-нибудь хотели знать что меняется от версии к версии? <br/>Вы сможете узнанать это в <a>Журнале изменений</a> на github!";
$older_versions_button = "Журнал изменений";
$reportbug = "Сообщить об ошибке";
$reportbug_text = "Вы нашли ошибку или у вас есть предложение как <a>улучшить</a> QupZilla?<br/>
                   Пожалуйста, сообщите нам об этом на трекере GitHub .";
$reportbug_button = "Сообщить сейчас";
$technology = "Технологии";
$technology_text = "QupZilla - это современный браузер использующий WebKit и Qt Framework. WebKit гарантирует <a>быстрый</a> 
                    просмотр веб-страниц, а Qt доступность на всех основных платформах.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Системный стиль";
$looknfeel_text = "QupZilla использует системный стиль элементов управления для основных окружений рабочего стола Linux. Так же, используются иконки из системной темы. А если стандартные темы наскучат вам вы всегда сможете поменять тему!.";
$library_header = "Единая библиотека";
$library_text = "QupZilla объединяет закладки, историю и RSS в одно удобное окно. Забудьте о множестве открытых окон, QupZilla использует только одно!<br/>
Благодаря встроенному RSS, вы сможете быть вкурсе последних изменений на ваших любимых сайтах. QupZilla также может импортировать закладки из других браузеров.";
$adblock_header = "Встроенный AdBlock";
$adblock_text = "Вы устали от сайтов с всплывающей рекламой? Они сжирают ваш траффик и время? Благодаря QupZilla вам нужно только обновить EasyList или добавить свои правила. Забудьте о рекламе!";
$speeddial_header = "Страница быстрого доступа";
$speeddial_text = "Это популярное расширение доступно для пользователей QupZilla! Теперь вы сможете заходить на ваши любимые страницы лишь за то время, которое потребуется на открытие новой вкладки. Страница быстрого доступа поддерживает drag&amp;drop и создание эскизов страниц.";
$devnews_header = "Новости разработки";
$feed_loading = "Загрузка...";

// Download Page
$other_linux = "Другой дистрибутив Linux";
$source_code = "Исходный код";
$choose_os = "Пожалуйста, выберите вашу операционную систему";
$windows_text = "32-х битный и 64-х битный установщик для Windows можно скачать кликнув по ссылке ниже";
$ubuntu_text = "Пользователи Precise, Oneiric, Natty и Maverick могут установить QupZilla выполнив эти команды";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "Версия для разработки:";
$packages_text = "32-х и 64-х битные пакеты";
$tarballs_text = "32-х и 64-х битные предварительно скомпилированные архивы";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "могут быть скачаны по ссылке ниже";
$source_text = "Вы можете получить исходный код, скопировав репозиторий ( если у вас установлен git)";
$source_text2 = "Также можно просмотреть его онлайн и скачать в архиве zip";
$view_source = "просмотреть исходный код на github.com";
$download_snapshot = "скачать текущий снимок";

// Contribute Page
$contribute_to = "Внести вклад в развитие QupZilla";
$contribute_text = "QupZilla проект с открытым исходным кодом, так что вы можете помочь его развитию! Я был бы очень рад включить в проект ваши патчи, которые вы пришлете мне. Но программирование 
		   не единственный способ помочь проекту: Вы можете перевести QupZilla на ваш язык или просто рассказать о нем вашим друзьям. Помните, любая (даже небольшая) помошь очень ценится нами!";
$getting_source = "Получить исходный код";
$getting_source_text1 = "Самый простой способ получить код QupZilla - скопировать его из репозитория github. Вы можете сделать это запустив эту комманду";
$getting_source_text2 = "затем вы сможете изменять код. Вы можете послать ваши патчи мне на e-mail или на github.";
$translating = "Перевод на другие языки";
$translating_text = "Другой способ помочь - перевести на другой язык или улучшить имеющийся перевод. Если вы хотите добавить новый язык, вы можете сгенерировать новый файл перевода вручную, или связаться со мной, и я сделаю это для вас. Вы можете послать ваш перевод по электронной почте или на github.";
$translating_moreinfo = "Получить больше информации о переводе можно <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>здесь</a> [wiki на github]";
$creators = "Кто автор QupZilla?";
$creators_text = "Основатель проекта, а также главный разработчик - студент из  Чехии <b>David Rosca</b> (nowrep).<br/>
Кроме программистов, другие люди внесли вклад сделав перевод или поддержав QupZilla. Полный список этих людей можно найти <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>здесь</a> [на github].<br/><br/>Также вы можете присоединиться к к IRC-каналу: <b>#qupzilla</b> на irc.freenode.net чтобы поболтать с людьми, которые приняли участие в развитии QupZilla.";
$share_with_friends = "Рассказать друзьям!";
$share_with_friends_text = "Вам понравилась QupZilla? Тогда расскажите своим друзьям о ней!";
?>
