<?php
// Header + Footer
$site_title = "QupZilla - легкий багатоплатформений браузер";
$qupzilla = "QupZilla";
$header_description = "Веб-переглядач";

$menu_home = "Головна";
$menu_screenshots = "Скріншоти";
$menu_download = "Скачати";
$menu_faq = "ЧаПи";
$menu_about = "Співробітництво";
$footer_site = "www.qupzilla.com";
$translated_by = "Сторінку переклав Кузнєцов Станіслав"; // "Сторінку переклав Кузнєцов Станіслав"

//Home Page
$actual_version = "Поточна версія";
$actual_version_text = "Поточна версія від ". $qupzilla_version_date .
					   "Це версія " . $qupzilla_version . ". Дізнатись про зміни можна в розділі \"журнал змін\". <br/>Не вагайтесь і <a href=\"download\">завантажуйте</a> зараз!";
$actual_version_button = "Завантажити";
$older_versions = "Журнал змін";
$older_versions_text = "Хочете знати що саме змінюється від версії до версії? <br/>Дізнавайтесь про це в <a>журналі змін</a> на github!";
$older_versions_button = "Журнал змін";
$reportbug = "Повідомити про помилку";
$reportbug_text = "Ви знайшли помилку або у Вас є пропозиції як <a>удосконалити</a> QupZill'у?<br/>
                  Будь-ласка, розкажіть про це на трекері github.";
$reportbug_button = "Повідомити зараз";
$technology = "Технологія";
$technology_text = "QupZilla - це сучасний веб-переглядач на базі ядра WebKit і Qt Framework. WebKit гарантує <a>швидкий</a> 
                    перегляд сторінок, а Qt - доступність на всіх основних платформах.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Звичний вигляд";
$looknfeel_text = "QupZilla використовує рідний стиль елементів керування основних середовищ Linux. А також іконки поточної теми. А якщо, раптом, такий вигляд Вам набридне, або у Вас виникнуть якісь проблеми - Ви завжди зможете змінити поточну тему на інші.";
$library_header = "Уніфікована бібліотека";
$library_text = "QupZilla об'єднує закладки, історію і rss-читалку в одному систематизованому вікні. Більше жодного засилля вікон, QupZilla використовує лише одне!<br/>
Користуючись вбудованою rss-читалкою Ви можете слідкувати за оновленнями Ваших улюблених сайтів. Також, Ви можете імпортувати до QupZill'и закладки з інших переглядачів.";
$adblock_header = "Вбудований AdBlock";
$adblock_text = "Вам набридли веб-сайти, заповнені рекламою? Вони пожирають Ваш трафік і час? Єдине, що Вам треба робити під час використання QupZill'и  - це оновлювати EasyList, або, можливо, додати власні правила і забути про рекламу.";
$speeddial_header = "Сторінка Speed Dial";
$speeddial_text = "Це популярне розширення нарешті доступне для користувачів QupZill'и! Тепер ви маєте доступ до Ваших улюблених сторінок лише за час, необхідний для відкриття нової вкладки. І, звісно, вона повністю підтримує drag&amp;drop і завантаження передпереглядів сторінок.";
$devnews_header = "Новини розробки";
$feed_loading = "Завантажується...";

// Download Page
$other_linux = "Інші Linux'и";
$source_code = "Початковий код";
$choose_os = "Будь ласка, виберіть Вашу операційну систему";
$windows_text = "32-хбітні та 64-хбітні встановлювачі для Windows можна завантажити клацнувши на нижчерозташовані посилання";
$ubuntu_text = "Користувачі Ubuntu Precise, Oneiric, Natty і Maverick можуть встановити QupZill'у запустивши наступні команди:";
$for_development_version = "Для версії, яка розробляється:";
$packages_text = "32-хбітні та 64-хбітні пакунки";
$tarballs_text = "32-хбітні та 64-хбітні скомпільовані архіви";
$qupzilla_for_text = "QupZill'у для";
$can_be_downloaded = "можна завантажити клацнувши нижчерозташовані посилання";
$source_text = "Ви можете отримати початковий код скопіювавши репозиторій (якщо у Вас встановлено git)";
$source_text2 = "Ви також можете переглянути його онлайн і скачати zip-архівом";
$view_source = "дивитись початковий код на github.com";
$download_snapshot = "скачати поточну копію";

// Contribute Page
$contribute_to = "Зробити внесок у розвиток QupZill'и";
$contribute_text = "QupZilla - це проект із відкритим початковим кодом, так що Ваша допомога завжди бажана! Я буду радий включити надіслані мені патчі. Але розробка коду - 
                    не єдиний спосіб допомогти: Ви можете перекласти QupZill'у Вашою мовою або просто поширити її серед друзів. Пам'ятайте, будь-яка (навіть, найменша) допомога цінується!";
$getting_source = "Отримання початковий код";
$getting_source_text1 = "Найлегший спосіб отримати початковий код QupZill'и - скопіювати з репозиторію github. Ви можете це зробити запустивши наступну команду";
$getting_source_text2 = "і тоді Ви можете почати кодувати. Ви можете висилати власні патчі мені на e-mail або до github'у.";
$translating = "Переклад іншими мовами";
$translating_text = "Інший спосіб допоогти - додати або покращити поточні переклади. Якщо Ви хочете додати нову мову, Ви можете згенерувати нові файли перекладів власноруч, або можете зв'язатись зі мною, і я зроблю це для Вас. Ви можете висилати Ваші переклади мені на e-mail або на github.";
$translating_moreinfo = "Більше інформації про переклад <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>тут</a> [wiki at github]";
$creators = "Хто розробляє QupZill'у?";
$creators_text = "Власником проекту, керівником і головним розробником є чеський студент <b>David Rosca</b> (nowrep).<br/>
Окрім програмування, інші учасники проекту допомагають перекладаючи або підтримуючи QupZill'у. Повний перелік співрозробників можна знайти <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>тут</a> [at github].<br/><br/>Також, Ви можете приєднатись до IRC-каналу: <b>#qupzilla</b> на irc.freenode.net, щоб поспілкуватись з людьми, які розробляють QupZill'у.";
$share_with_friends = "Поділись із друзями!";
$share_with_friends_text = "Вам подобається QupZilla? Тоді поширюйте її серед друзів!";
?>
