/* ============================================================
* QupZilla - WebKit based browser
* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* ============================================================ */
#ifndef BOOKMARKICON_H
#define BOOKMARKICON_H

#include <QUrl>

#include "clickablelabel.h"
#include "bookmarksmodel.h"
#include "qz_namespace.h"

class SpeedDial;
class QupZilla;

class QT_QUPZILLA_EXPORT BookmarkIcon : public ClickableLabel
{
    Q_OBJECT
public:
    explicit BookmarkIcon(QupZilla* mainClass, QWidget* parent = 0);
    void checkBookmark(const QUrl &url, bool forceCheck = false);

private slots:
    void bookmarkAdded(const BookmarksModel::Bookmark &bookmark);
    void bookmarkDeleted(const BookmarksModel::Bookmark &bookmark);
    void speedDialChanged();

private:
    void contextMenuEvent(QContextMenuEvent* ev);
    void mousePressEvent(QMouseEvent* ev);

    void setBookmarkSaved();
    void setBookmarkDisabled();

    QupZilla* p_QupZilla;
    BookmarksModel* m_bookmarksModel;
    SpeedDial* m_speedDial;

    QUrl m_lastUrl;

};

#endif // BOOKMARKICON_H
