<?php
// Header + Footer
$site_title = "QupZilla - Peramban multiplatform ringan";
$qupzilla = "QupZilla";
$header_description = "Peramban Web";

$menu_home = "Rumah";
$menu_screenshots = "Screenshots";
$menu_download = "Unduhan";
$menu_faq = "FAQ";
$menu_about = "Berkontribusi";
$footer_site = "www.qupzilla.com";
$translated_by = "Halaman diterjemahkan oleh Widya Walesa"; // "Page translated by <translator's name>"

//Home Page
$actual_version = "Versi Terakhir";
$actual_version_text = "Versi terakhir adalah pada ". $qupzilla_version_date .
					   " dengan versi " . $qupzilla_version . ". Perubahan dapat dilihat di changelog. <br/>Jangan ragu serta <a href=\"download\">unduh</a> QupZilla saat ini juga!";
$actual_version_button = "UNDUH";
$older_versions = "Changelog";
$older_versions_text = "Anda ingin mengetahui perubahan apa saja yang terjadi dari versi tersebut hingga saat ini? <br/>Anda dapat melihatnya di <a>changelog</a> di github!";
$older_versions_button = "Changelog";
$reportbug = "Laporkan Isu";
$reportbug_text = "Apakah anda menemukan kesalahan atau punya ide dan saran untuk <a>meningkatkan</a> QupZilla?<br/>
                  Silakan kirim hal tersebut ke penelusuran isu di github.";
$reportbug_button = "Laporkan Sekarang";
$technology = "Teknologi";
$technology_text = "QupZilla adalah peramban web modern yang berbasis pada WebKit dan Framework Qt. WebKit menjamin
                    perambahan <a>cepat</a> dan ketersediaan Qt pada semua platform utama.";
$technology_button = "WebKit &amp; Qt";
$looknfeel_header = "Tampilan Asli";
$looknfeel_text = "QupZilla menggunakan gaya widget asli dari Lingkungan Desktop Linux yang utama. QupZilla juga menggunakan tema icon desktop yang sedang digunakan. Jika anda merasa bosan dengan tema asli atau bermasalah dengan tema asli, anda dapat menggantinya dengan tema yang lain.";
$library_header = "Perpustakaan Tunggal";
$library_text = "QupZilla menyatukan bookmark, sejarah dan pembaca rss dalam satu jendela yang tertata rapi. Tak perlu banyak jendela, QupZilla hanya menggunakan satu jendela saja!<br/>
Dengan pembaca rss yang terintegrasi, anda dapat mengikuti hal-hal baru di situs kesayangan anda. QupZilla juga dapat mengimpor bookmark dari peramban yang lain.";
$adblock_header = "AdBlock Terintegrasi";
$adblock_text = "Anda bosan dengan situs web yang penuh dengan iklan? Apakah mereka menghabiskan bandwidth dan waktu anda? Yang perlu anda lakukan di QupZilla adalah memperbarui EasyList atau mungkin menambahkan aturan anda sendiri dan mulailah perambahan bebas iklan.";
$speeddial_header = "Panggilan Cepat";
$speeddial_text = "Ekstensi populer ini akhirnya tersedia untuk pengguna QupZilla! Anda dapat mengakses halaman kesayangan anda secepat anda membuka satu halaman di tab baru. Tak perlu dikatakan lagi bahwa panggilan cepat mendukung drag&amp;drop dan pemuatan thumbnail halaman.";
$devnews_header = "Berita Pengembangan";
$feed_loading = "Memuat...";

// Download Page
$other_linux = "Linux Lainnya";
$source_code = "Kode Sumber";
$choose_os = "Silakan pilih sistem operasi anda";
$windows_text = "Instalatur Windows 32bit dan 64bit dapat diunduh dengan mengklik tautan di bawah ini";
$ubuntu_text = "Pengguna Precise, Oneiric, Natty dan Maverick dapat menginstal QupZilla dengan menjalankan perintah berikut ini";
$install_run_command = "You can install QupZilla by running this command";
$for_development_version = "untuk versi pengembangan:";
$packages_text = "Paket 32bit dan 64bit";
$tarballs_text = "Prakompilasi Tarball 32bit dan 64bit";
$qupzilla_for_text = "QupZilla for";
$can_be_downloaded = "dapat diunduh dengan mengklik tautan di bawah ini";
$source_text = "Anda dapat mengunduh kode sumber dengan mengkloning repositori (jika anda sudah menginstal git)";
$source_text2 = "Anda juga dapat melihatnya secara online dan mengunduhnya dalam arsip zip";
$view_source = "lihat kode sumber di github.com";
$download_snapshot = "unduh snapshot terakhir";

// Contribute Page
$contribute_to = "Berkontribusi ke QupZilla";
$contribute_text = "QupZilla adalah proyek open source, jadi bantuan anda sangat dinantikan! Saya akan sangat senang untuk mengikutsertakan perbaikan yang anda kirimkan. Tetapi membongkar
                    kode bukanlah satu-satunya cara untuk membantu: Anda dapat menterjemahkan QupZilla ke dalam bahasa anda atau cukup dengan membaginya ke teman anda. Ingatlah, setiap bantuan (meskipun kecil) adalah sangat berarti!";
$getting_source = "Memperoleh Kode Sumber";
$getting_source_text1 = "Cara termudah untuk memperoleh kode sumber QupZilla adalah dengan mengkloningnya dari repositori github. Anda dapat melakukannya dengan menjalankan perintah ini";
$getting_source_text2 = "dan anda dapat langsung membongkarnya. Anda dapat mengirimkan perbaikan anda melalui surel atau di github.";
$translating = "Menterjemahkan ke bahasa yang lain";
$translating_text = "Cara lain untuk berkontribusi adalah menambah atau meningkatkan terjemahan saat ini. Jika anda ingin menambahkan bahasa baru, anda dapat menyusun berkas terjemahan baru secara manual, atau anda dapat menghubungi saya, dan saya akan melakukannya untuk anda. Anda dapat mengirimkan terjemahan anda kembali melalui email atau di github.";
$translating_moreinfo = "Informasi lanjutan tentang penterjemahan <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>di sini</a> [wiki di github]";
$creators = "Siapa yang membuat QupZilla?";
$creators_text = "Pemilik proyek, pengelola dan pengembang utama adalah pelajar Czech <b>David Rosca</b> (nowrep).<br/>
Selain dari pengkodean, pihak lain juga berkontribusi dengan membuat terjemahan atau mendukung QupZilla. Daftar lengkap dari kontributor dapat dilihat di <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>sini</a> [di github].<br/><br/>Anda juga dapat bergabung di kanal IRC <b>#qupzilla</b> di irc.freenode.net untuk berbincang dengan orang-orang yang terlibat di QupZilla.";
$share_with_friends = "Bagi Dengan Teman!";
$share_with_friends_text = "Anda menyukai QupZilla? Bagikan pada teman-teman anda!";
?>
