<?php
// Header + Footer
$site_title = "QupZilla - Navegador web multi-plataforma";
$qupzilla = "QupZilla";
$header_description = "Navegador web";

$menu_home = "Página inicial";
$menu_screenshots = "Imagens";
$menu_download = "Transferir";
$menu_faq = "FAQ";
$menu_about = "Participar";
$footer_site = "www.qupzilla.com";
$translated_by = "Página traduzida por Sérgio Marques";

//Home Page
$actual_version = "Versão atual";
$actual_version_text = "A versão atual data de ". $qupzilla_version_date .
					   " e é a versão " . $qupzilla_version . ". As novidades estão no registo de alterações.<br/>Não perca tempo e <a href=\"download\">transfira</a> já!";
$actual_version_button = "Transferir";
$older_versions = "Registo de alterações";
$older_versions_text = "Pretende saber que alterações foram efetuadas entre as versões? <br/>Encontra-as no <a>registo de alterações</a> do código fonte!";
$older_versions_button = "Registo de alterações";
$reportbug = "Reportar erros";
$reportbug_text = "Encontrou erros ou pretende sugerir algo para <a>melhorar</a> o QupZilla?<br/> Crie um relatório no github.";
$reportbug_button = "Reportar agora";
$technology = "Tecnologia";
$technology_text = "O QupZilla é um navegador web baseado no WebKit e na plataforma Qt. O WebKit garante uma navegação <a>rápida</a> e a plataforma Qt disponibiliza a tecnologia para os principais sistemas operativos.";
$technology_button = "WebKit e Qt";
$looknfeel_header = "Aparência de acordo com o sistema operativo";
$looknfeel_text = "O QupZilla utiliza o estilo e os ícones dos ambientes de trabalho em que for instalado. No entanto, se considerar que este aspeto é aborrecido ou caso tenha alguns problemas, pode sempre trocar entre os temas disponíveis.";
$library_header = "Agregação de componentes";
$library_text = "O QupZilla agrega os marcadores, histórico e fontes RSS numa só janela. Nada de diversas janelas para estes itens!<br/> Com o leitor RSS integrado, tem sempre as suas fontes atualizadas. Para além disso, o QupZilla consegue importar os marcadores de outros navegadores web.";
$adblock_header = "AdBlock integrado";
$adblock_text = "Não gosta dos anúncios que algumas páginas web exibem? Estão estes anúncios a gastar largura de banda e fazem-no(a) perder tempo? Com o QupZilla, tudo o que tem de fazer é atualizar a EasyList ou, eventualmente, adicionar as suas regras e pode navegar sem ser importunado.";
$speeddial_header = "Ligação rápida";
$speeddial_text = "Finalmente, este funcionalidade está disponível para os utilizadores QupZilla! Agora já pode aceder, fácil e rapidamente, às suas páginas favoritas com apenas um clique. As miniaturas das páginas são carregadas automaticamente e pode utilizar o processo arrastar e largar para as mover entre as diversas posições.";
$devnews_header = "Desenvolvimento";
$feed_loading = "A carregar...";

// Download Page
$other_linux = "Outros sistemas linux";
$source_code = "Código fonte";
$choose_os = "Escolha o seu sistema operativo";
$windows_text = "Pode transferir as versões 32 bits e 64 bits para o Windows nas ligações abaixo";
$ubuntu_text = "Os utilizadores do Ubuntu Precise, Oneiric, Natty e Maverick podem instalar o QupZilla através destes comandos";
$install_run_command = "Pode intalar o QupZilla com este comando";
$for_development_version = "Versões de desenvolvimento:";
$packages_text = "Os pacotes 32 bits e 64 bits";
$tarballs_text = "Os pacotes pré-compilados 32 bits e 64 bits";
$qupzilla_for_text = "QupZilla para";
$can_be_downloaded = "podem ser transferidos nas ligações abaixo";
$source_text = "Pode obter o código fonte clonando o repositório (se tiver o git instalado)";
$source_text2 = "Também pode ver o código fonte no github e/ou transferir o código no formato .zip";
$view_source = "ver código fonte em github.com";
$download_snapshot = "transferir código fonte atual";

// Contribute Page
$contribute_to = "Contribuir no desenvolvimento do QupZilla";
$contribute_text = "O QupZilla é um programa livre, logo toda a ajuda é aceite! Terei muito gosto em incluir as correções que me enviarem. No entanto, esta não é a única forma de ajudar. Também pode traduzir o QupZilla no seu idioma, publicitar e/ou partilhar o programa. Lembre-se que qualquer ajuda, mesmo que pequena é importante e altamente apreciada!";
$getting_source = "Obter o código fonte";
$getting_source_text1 = "O método mais fácil de obter o código fonte é clonar o repositório github. Pode clonar o 	repositório com este comando";
$getting_source_text2 = "e já pode começar a mexer no código. Pode enviar as suas correções por correio eletrónico ou 	através do github.";
$translating = "Traduzir o programa";
$translating_text = "Outra forma de contribuir no desenvolvimento é adicionando ou corrigindo as traduções existentes. Se quiser adicionar um novo idioma, pode gerar o ficheiro manualmente ou contactar-me para que eu o crie por si. Posteriormente, pode enviar o ficheiro traduzido por correio eletrónico ou através do github.";
$translating_moreinfo = "Pode encontrar mais informações sobre o processo de traduções <a href='https://github.com/QupZilla/qupzilla/wiki/Translating'>aqui</a> [wiki no github]";
$creators = "Quem é o programador do QupZilla?";
$creators_text = "O criador e programador principal do Qupzilla é um estudante Checo de nome <b>David Rosca</b> (nowrep).<br/>Para além do programador principal, outras pessoas ajudam no desenvolvimento, tradução e apoio ao QupZilla. Pode encontrar a lista de contributos <a href='https://github.com/QupZilla/qupzilla/blob/master/AUTHORS'>aqui</a> [no github].<br/><br/>Você também pode ajudar. Aceda ao canal IRC do <b>#qupzilla</b> em irc.freenode.net e converse com as pessoas envolvidas no desenvolvimento do QupZilla.";
$share_with_friends = "Partilhar com os amigos!";
$share_with_friends_text = "Gosta do QupZilla? Então, partilhe-o com os seus amigos!";
?>
